@@ -868,13 +868,18 @@ public:
     unsigned long index() const;
     QString toHTML();
     void applyChanges();
-    void getCursor(int offset, int &_x, int &_y, int &height);
     /**
      * not part of the DOM.
      * @returns the exact coordinates and size of this element.
      */
     QRect getRect();
 
+    /**
+     * not part of the DOM.
+     * @returns whether this node is contenteditable.
+     */
+    bool isContentEditable() const;
+
 protected:
     NodeImpl *impl;
 };
