@@ -4,7 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2003 Apple Computer, Inc.
+ * Copyright (C) 2004 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,6 +31,7 @@
 #include "khtmlview.h"
 #include "khtml_part.h"
 #include "html/html_documentimpl.h"
+#include "html_imageimpl.h"
 #include "khtml_settings.h"
 #include "misc/htmlhashes.h"
 
@@ -322,9 +323,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
     if(!codec)
         codec = QTextCodec::codecForLocale();
 
-#if !APPLE_CHANGES
     QStringList fileUploads;
-#endif
 
     for (QPtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
         HTMLGenericFormElementImpl* current = it.current();
@@ -369,22 +368,30 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
                         static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::FILE)
                     {
                         QString path = static_cast<HTMLInputElementImpl*>(current)->value().string();
-#if !APPLE_CHANGES
                         if (path.length()) fileUploads << path;
-#endif
+
+                        // Turn non-ASCII characters into &-escaped form.
+                        // This doesn't work perfectly, because it doesn't escape &, for example.
+                        // But it seems to be what Gecko does.
+                        QString onlyfilename;
+                        for (uint i = path.findRev('/') + 1; i < path.length(); ++i) {
+                            QChar c = path.at(i).unicode();
+                            if (c.unicode() >= 0x20 && c.unicode() <= 0x7F) {
+                                onlyfilename.append(&c, 1);
+                            } else {
+                                QString ampersandEscape;
+                                ampersandEscape.sprintf("&#%hu;", c.unicode());
+                                onlyfilename.append(ampersandEscape);
+                            }
+                        }
 
                         // FIXME: This won't work if the filename includes a " mark,
-                        // or control characters like CR or LF. This also does strange
-                        // things if the filename includes characters you can't encode
-                        // in the website's character set.
-                        hstr += "; filename=\"";
-                        hstr += codec->fromUnicode(path.mid(path.findRev('/') + 1));
-                        hstr += "\"";
-
+                        // or control characters like CR or LF.
+                        hstr += ("; filename=\"" + onlyfilename + "\"").ascii();
                         if(!static_cast<HTMLInputElementImpl*>(current)->value().isEmpty())
                         {
 #if APPLE_CHANGES
-                            QString mimeType = part ? KWQ(part)->mimeTypeForFileName(path) : QString();
+                            QString mimeType = part ? KWQ(part)->mimeTypeForFileName(onlyfilename) : QString();
 #else
                             KMimeType::Ptr ptr = KMimeType::findByURL(KURL(path));
                             QString mimeType = ptr->name();
@@ -515,9 +522,12 @@ void HTMLFormElementImpl::submit( bool activateSubmitButton )
         if (current->id() == ID_INPUT) {
             HTMLInputElementImpl *input = static_cast<HTMLInputElementImpl*>(current);
             if (input->inputType() == HTMLInputElementImpl::TEXT
-                || input->inputType() ==  HTMLInputElementImpl::PASSWORD)
+                || input->inputType() ==  HTMLInputElementImpl::PASSWORD
+                || input->inputType() == HTMLInputElementImpl::SEARCH)
             {
                 KWQ(part)->recordFormValue(input->name().string(), input->value().string(), this);
+                if (input->renderer() && input->inputType() == HTMLInputElementImpl::SEARCH)
+                    static_cast<RenderLineEdit*>(input->renderer())->addSearchResult();
             }
         }
 #else
@@ -589,7 +599,7 @@ void HTMLFormElementImpl::reset(  )
     m_inreset = false;
 }
 
-void HTMLFormElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLFormElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
@@ -664,7 +674,7 @@ void HTMLFormElementImpl::parseAttribute(AttributeImpl *attr)
 	}
 	// fall through
     default:
-        HTMLElementImpl::parseAttribute(attr);
+        HTMLElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -690,6 +700,11 @@ void HTMLFormElementImpl::removeFormElement(HTMLGenericFormElementImpl *e)
     formElements.remove(e);
 }
 
+bool HTMLFormElementImpl::isURLAttribute(AttributeImpl *attr) const
+{
+    return attr->id() == ATTR_ACTION;
+}
+
 // -------------------------------------------------------------------------
 
 HTMLGenericFormElementImpl::HTMLGenericFormElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
@@ -712,24 +727,24 @@ HTMLGenericFormElementImpl::~HTMLGenericFormElementImpl()
         m_form->removeFormElement(this);
 }
 
-void HTMLGenericFormElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLGenericFormElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
     case ATTR_NAME:
         break;
     case ATTR_DISABLED:
-        setDisabled( attr->val() != 0 );
+        setDisabled( !attr->isNull() );
         break;
     case ATTR_READONLY:
     {
         bool m_oldreadOnly = m_readOnly;
-        m_readOnly = attr->val() != 0;
+        m_readOnly = !attr->isNull();
         if (m_oldreadOnly != m_readOnly) setChanged();
         break;
     }
     default:
-        HTMLElementImpl::parseAttribute(attr);
+        HTMLElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -769,6 +784,15 @@ void HTMLGenericFormElementImpl::attach()
     }
 }
 
+void HTMLGenericFormElementImpl::removedFromDocument()
+{
+    if (m_form)
+        m_form->removeFormElement(this);
+    m_form = 0;
+   
+    HTMLElementImpl::removedFromDocument();
+}
+
 HTMLFormElementImpl *HTMLGenericFormElementImpl::getForm() const
 {
     NodeImpl *p = parentNode();
@@ -851,8 +875,7 @@ bool HTMLGenericFormElementImpl::isKeyboardFocusable() const
     if (isFocusable()) {
         if (m_render->isWidget()) {
             return static_cast<RenderWidget*>(m_render)->widget() &&
-            ((static_cast<RenderWidget*>(m_render)->widget()->focusPolicy() == QWidget::TabFocus) ||
-             (static_cast<RenderWidget*>(m_render)->widget()->focusPolicy() == QWidget::StrongFocus));
+                (static_cast<RenderWidget*>(m_render)->widget()->focusPolicy() & QWidget::TabFocus);
         }
 	if (getDocument()->part())
 	    return getDocument()->part()->tabsToAllControls();
@@ -865,8 +888,7 @@ bool HTMLGenericFormElementImpl::isMouseFocusable() const
     if (isFocusable()) {
         if (m_render->isWidget()) {
             return static_cast<RenderWidget*>(m_render)->widget() &&
-            ((static_cast<RenderWidget*>(m_render)->widget()->focusPolicy() == QWidget::ClickFocus) ||
-             (static_cast<RenderWidget*>(m_render)->widget()->focusPolicy() == QWidget::StrongFocus));
+                (static_cast<RenderWidget*>(m_render)->widget()->focusPolicy() & QWidget::ClickFocus);
         }
 #if APPLE_CHANGES
         // For <input type=image> and <button>, we will assume no mouse focusability.  This is
@@ -891,21 +913,6 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
             if (ext)
                 ext->editableWidgetFocused(widget);
         }
-        if (evt->id()==EventImpl::MOUSEDOWN_EVENT || evt->id()==EventImpl::KEYDOWN_EVENT)
-        {
-            setActive();
-        }
-        else if (evt->id() == EventImpl::MOUSEUP_EVENT || evt->id()==EventImpl::KEYUP_EVENT)
-        {
-	    if (m_active)
-	    {
-		setActive(false);
-		setFocus();
-	    }
-	    else {
-                setActive(false);
-            }
-        }
 
 #if APPLE_CHANGES
 	// We don't want this default key event handling, we'll count on
@@ -1022,7 +1029,7 @@ DOMString HTMLButtonElementImpl::type() const
     return getAttribute(ATTR_TYPE);
 }
 
-void HTMLButtonElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLButtonElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
@@ -1049,7 +1056,7 @@ void HTMLButtonElementImpl::parseAttribute(AttributeImpl *attr)
             getDocument()->createHTMLEventListener(attr->value().string()));
         break;
     default:
-        HTMLGenericFormElementImpl::parseAttribute(attr);
+        HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -1153,7 +1160,7 @@ RenderObject* HTMLFieldSetElementImpl::createRenderer(RenderArena* arena, Render
 // -------------------------------------------------------------------------
 
 HTMLInputElementImpl::HTMLInputElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
-    : HTMLGenericFormElementImpl(doc, f)
+    : HTMLGenericFormElementImpl(doc, f), m_imageLoader(0)
 {
     m_type = TEXT;
     m_maxLen = -1;
@@ -1170,6 +1177,10 @@ HTMLInputElementImpl::HTMLInputElementImpl(DocumentPtr *doc, HTMLFormElementImpl
     xPos = 0;
     yPos = 0;
 
+#if APPLE_CHANGES
+    m_maxResults = 0;
+#endif
+
     if ( m_form )
         m_autocomplete = f->autoComplete();
 }
@@ -1177,6 +1188,7 @@ HTMLInputElementImpl::HTMLInputElementImpl(DocumentPtr *doc, HTMLFormElementImpl
 HTMLInputElementImpl::~HTMLInputElementImpl()
 {
     if (getDocument()) getDocument()->deregisterMaintainsState(this);
+    delete m_imageLoader;
 }
 
 NodeImpl::Id HTMLInputElementImpl::id() const
@@ -1208,6 +1220,12 @@ void HTMLInputElementImpl::setType(const DOMString& t)
         newType = BUTTON;
     else if ( strcasecmp( t, "khtml_isindex" ) == 0 )
         newType = ISINDEX;
+#if APPLE_CHANGES
+    else if ( strcasecmp( t, "search" ) == 0 )
+        newType = SEARCH;
+    else if ( strcasecmp( t, "range" ) == 0 )
+        newType = RANGE;
+#endif
     else
         newType = TEXT;
 
@@ -1217,7 +1235,7 @@ void HTMLInputElementImpl::setType(const DOMString& t)
     if (m_type != newType) {
         if (newType == FILE && m_haveType) {
             // Set the attribute back to the old value.
-            // Useful in case we were called from inside parseAttribute.
+            // Useful in case we were called from inside parseHTMLAttribute.
             setAttribute(ATTR_TYPE, type());
         } else {
             m_type = newType;
@@ -1240,6 +1258,10 @@ DOMString HTMLInputElementImpl::type() const
     case HIDDEN: return "hidden";
     case IMAGE: return "image";
     case BUTTON: return "button";
+#if APPLE_CHANGES
+    case SEARCH: return "search";
+    case RANGE: return "range";
+#endif
     default: return "";
     }
 }
@@ -1321,6 +1343,10 @@ void HTMLInputElementImpl::click()
         case IMAGE:
         case ISINDEX:
         case PASSWORD:
+#if APPLE_CHANGES
+        case SEARCH:
+        case RANGE:
+#endif
         case TEXT:
             HTMLGenericFormElementImpl::click();
             break;
@@ -1335,6 +1361,9 @@ void HTMLInputElementImpl::accessKeyAction()
             break;
         case TEXT:
         case PASSWORD:
+#if APPLE_CHANGES
+        case SEARCH:
+#endif
         case ISINDEX:
             focus();
             break;
@@ -1345,6 +1374,9 @@ void HTMLInputElementImpl::accessKeyAction()
         case IMAGE:
         case BUTTON:
         case FILE:
+#if APPLE_CHANGES
+        case RANGE:
+#endif
             // focus and click
             focus();
             click();
@@ -1352,7 +1384,26 @@ void HTMLInputElementImpl::accessKeyAction()
     }
 }
 
-void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
+bool HTMLInputElementImpl::mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const
+{
+    switch (attr) {
+        case ATTR_WIDTH:
+        case ATTR_HEIGHT:
+        case ATTR_VSPACE:
+        case ATTR_HSPACE:
+            result = eUniversal;
+            return false;
+        case ATTR_ALIGN:
+            result = eReplaced; // Share with <img> since the alignment behavior is the same.
+            return false;
+        default:
+            break;
+    }
+    
+    return HTMLElementImpl::mapToEntry(attr, result);
+}
+
+void HTMLInputElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
@@ -1361,52 +1412,60 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_TYPE:
         setType(attr->value());
+        if (m_type != IMAGE && m_imageLoader) {
+            delete m_imageLoader;
+            m_imageLoader = 0;
+        }
         break;
     case ATTR_VALUE:
         if (m_value.isNull()) // We only need to setChanged if the form is looking
             setChanged();     // at the default value right now.
         break;
     case ATTR_CHECKED:
-        m_defaultChecked = attr->val() != 0;
+        m_defaultChecked = !attr->isNull();
         if (m_useDefaultChecked) {
             setChecked(m_defaultChecked);
             m_useDefaultChecked = true;
         }
         break;
     case ATTR_MAXLENGTH:
-        m_maxLen = attr->val() ? attr->val()->toInt() : -1;
+        m_maxLen = !attr->isNull() ? attr->value().toInt() : -1;
         setChanged();
         break;
     case ATTR_SIZE:
-        m_size = attr->val() ? attr->val()->toInt() : 20;
+        m_size = !attr->isNull() ? attr->value().toInt() : 20;
         break;
     case ATTR_ALT:
+        if (m_render && m_type == IMAGE)
+            static_cast<RenderImage*>(m_render)->updateAltText();
+        break;
     case ATTR_SRC:
-        if (m_render && m_type == IMAGE) m_render->updateFromElement();
+        if (m_render && m_type == IMAGE) {
+            if (!m_imageLoader)
+                m_imageLoader = new HTMLImageLoader(this);
+            m_imageLoader->updateFromElement();
+        }
         break;
     case ATTR_USEMAP:
     case ATTR_ACCESSKEY:
         // ### ignore for the moment
         break;
     case ATTR_VSPACE:
-        addCSSLength(CSS_PROP_MARGIN_TOP, attr->value());
-        addCSSLength(CSS_PROP_MARGIN_BOTTOM, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_TOP, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_BOTTOM, attr->value());
         break;
     case ATTR_HSPACE:
-        addCSSLength(CSS_PROP_MARGIN_LEFT, attr->value());
-        addCSSLength(CSS_PROP_MARGIN_RIGHT, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_LEFT, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_RIGHT, attr->value());
         break;        
     case ATTR_ALIGN:
-        addHTMLAlignment( attr->value() );
+        addHTMLAlignment(attr);
         break;
     case ATTR_WIDTH:
-        // ignore this attribute,  do _not_ add
-        // a CSS_PROP_WIDTH here (we will honor this attribute for input type=image
-        // in the attach call)!
-        // webdesigner are stupid - and IE/NS behave the same ( Dirk )
+        addCSSLength(attr, CSS_PROP_WIDTH, attr->value() );
         break;
     case ATTR_HEIGHT:
-        addCSSLength(CSS_PROP_HEIGHT, attr->value() );
+        addCSSLength(attr, CSS_PROP_HEIGHT, attr->value() );
         break;
     case ATTR_ONFOCUS:
         setHTMLEventListener(EventImpl::FOCUS_EVENT,
@@ -1424,8 +1483,31 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
         setHTMLEventListener(EventImpl::CHANGE_EVENT,
             getDocument()->createHTMLEventListener(attr->value().string()));
         break;
+    case ATTR_ONINPUT:
+        setHTMLEventListener(EventImpl::INPUT_EVENT,
+                             getDocument()->createHTMLEventListener(attr->value().string()));
+        break;
+#if APPLE_CHANGES
+    // Search field and slider attributes all just cause updateFromElement to be called through style
+    // recalcing.
+    case ATTR_ONSEARCH:
+        setHTMLEventListener(EventImpl::SEARCH_EVENT,
+                             getDocument()->createHTMLEventListener(attr->value().string()));
+        break;
+    case ATTR_RESULTS:
+        m_maxResults = !attr->isNull() ? attr->value().toInt() : 0;
+        /* Fall through */
+    case ATTR_AUTOSAVE:
+    case ATTR_INCREMENTAL:
+    case ATTR_PLACEHOLDER:
+    case ATTR_MIN:
+    case ATTR_MAX:
+    case ATTR_PRECISION:
+        setChanged();
+        break;
+#endif
     default:
-        HTMLGenericFormElementImpl::parseAttribute(attr);
+        HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -1435,6 +1517,10 @@ bool HTMLInputElementImpl::rendererIsNeeded(RenderStyle *style)
     {
     case TEXT:
     case PASSWORD:
+#if APPLE_CHANGES
+    case SEARCH:
+    case RANGE:
+#endif
     case ISINDEX:
     case CHECKBOX:
     case RADIO:
@@ -1455,6 +1541,9 @@ RenderObject *HTMLInputElementImpl::createRenderer(RenderArena *arena, RenderSty
     {
     case TEXT:
     case PASSWORD:
+#if APPLE_CHANGES
+    case SEARCH:
+#endif
     case ISINDEX:  return new (arena) RenderLineEdit(this);
     case CHECKBOX: return new (arena) RenderCheckBox(this);
     case RADIO:    return new (arena) RenderRadioButton(this);
@@ -1463,6 +1552,9 @@ RenderObject *HTMLInputElementImpl::createRenderer(RenderArena *arena, RenderSty
     case RESET:    return new (arena) RenderResetButton(this);
     case FILE:     return new (arena) RenderFileButton(this);
     case BUTTON:   return new (arena) RenderPushButton(this);
+#if APPLE_CHANGES
+    case RANGE:    return new (arena) RenderSlider(this);
+#endif
     case HIDDEN:   break;
     }
     assert(false);
@@ -1489,23 +1581,31 @@ void HTMLInputElementImpl::attach()
                 setAttribute(ATTR_VALUE, nvalue);
         }
 
-        m_defaultChecked = (getAttribute(ATTR_CHECKED) != 0);
+        m_defaultChecked = (!getAttribute(ATTR_CHECKED).isNull());
         
         m_inited = true;
     }
 
-    switch( m_type ) {
-    case HIDDEN:
-    case IMAGE:
-        if (!getAttribute(ATTR_WIDTH).isEmpty())
-            addCSSLength(CSS_PROP_WIDTH, getAttribute(ATTR_WIDTH));
-        break;
-    default:
-        break;
+    // Disallow the width attribute on inputs other than HIDDEN and IMAGE.
+    // Dumb Web sites will try to set the width as an attribute on form controls that aren't
+    // images or hidden.
+    if (hasMappedAttributes() && m_type != HIDDEN && m_type != IMAGE && !getAttribute(ATTR_WIDTH).isEmpty()) {
+        int excCode;
+        removeAttribute(ATTR_WIDTH, excCode);
     }
 
     HTMLGenericFormElementImpl::attach();
 
+    if (m_type == IMAGE) {
+        if (!m_imageLoader)
+            m_imageLoader = new HTMLImageLoader(this);
+        m_imageLoader->updateFromElement();
+        if (renderer()) {
+            RenderImage* imageObj = static_cast<RenderImage*>(renderer());
+            imageObj->setImage(m_imageLoader->image());    
+        }
+    }
+
 #if APPLE_CHANGES
     // note we don't deal with calling passwordFieldRemoved() on detach, because the timing
     // was such that it cleared our state too early
@@ -1569,6 +1669,10 @@ bool HTMLInputElementImpl::encoding(const QTextCodec* codec, khtml::encodingList
     switch (m_type) {
         case HIDDEN:
         case TEXT:
+#if APPLE_CHANGES
+        case SEARCH:
+        case RANGE:
+#endif
         case PASSWORD:
             // always successful
             encoding += fixUpfromUnicode(codec, value().string());
@@ -1803,54 +1907,51 @@ void HTMLInputElementImpl::defaultEventHandler(EventImpl *evt)
 #if APPLE_CHANGES
     // Use key press event here since sending simulated mouse events
     // on key down blocks the proper sending of the key press event.
-    if (evt->id() == EventImpl::KEYPRESS_EVENT) {
-    
-        if (!m_form || !m_render || !evt->isKeyboardEvent())
-            return;
-        
+    if (evt->id() == EventImpl::KEYPRESS_EVENT && evt->isKeyboardEvent()) {
         DOMString key = static_cast<KeyboardEventImpl *>(evt)->keyIdentifier();
-        
         switch (m_type) {
+            case BUTTON:
+            case CHECKBOX:
+            case FILE:
             case IMAGE:
+            case RADIO:
             case RESET:
             case SUBMIT:
-                // simulate mouse click for spacebar and enter
+                // Simulate mouse click for enter or spacebar for these types of elements.
+                // The AppKit already does this for spacebar for some, but not all, of them.
                 if (key == "U+000020" || key == "Enter") {
-                    m_form->submitClick();
-                    evt->setDefaultHandled();
-                }
-                break;
-            case CHECKBOX:
-            case RADIO:
-                // for enter, find the first successful image or submit element 
-                // send it a simulated mouse click
-                if (key == "Enter") {
-                    m_form->submitClick();
+                    click();
                     evt->setDefaultHandled();
                 }
                 break;
+            case HIDDEN:
+            case ISINDEX:
+            case PASSWORD:
+            case RANGE:
+            case SEARCH:
             case TEXT:
-            case PASSWORD: {
-                // For enter, find the first successful image or submit element 
-                // send it a simulated mouse click.
-                if (key == "Enter") {
+                // Simulate mouse click on the default form button for enter for these types of elements.
+                if (key == "Enter" && m_form) {
                     m_form->submitClick();
                     evt->setDefaultHandled();
                 }
                 break;
-            }
-            default:
-                // not handled for the other widgets
-                break;
         }
     }
 #endif
+
     HTMLGenericFormElementImpl::defaultEventHandler(evt);
 }
 
 bool HTMLInputElementImpl::isEditable()
 {
-    return ((m_type == TEXT) || (m_type == PASSWORD) || (m_type == ISINDEX) || (m_type == FILE));
+    return ((m_type == TEXT) || (m_type == PASSWORD) ||
+            (m_type == SEARCH) || (m_type == ISINDEX) || (m_type == FILE));
+}
+
+bool HTMLInputElementImpl::isURLAttribute(AttributeImpl *attr) const
+{
+    return (attr->id() == ATTR_SRC);
 }
 
 // -------------------------------------------------------------------------
@@ -1874,7 +1975,7 @@ NodeImpl::Id HTMLLabelElementImpl::id() const
     return ID_LABEL;
 }
 
-void HTMLLabelElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLLabelElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
@@ -1887,7 +1988,7 @@ void HTMLLabelElementImpl::parseAttribute(AttributeImpl *attr)
             getDocument()->createHTMLEventListener(attr->value().string()));
         break;
     default:
-        HTMLElementImpl::parseAttribute(attr);
+        HTMLElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -1933,7 +2034,7 @@ DOMString HTMLLegendElementImpl::type() const
 // -------------------------------------------------------------------------
 
 HTMLSelectElementImpl::HTMLSelectElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
-    : HTMLGenericFormElementImpl(doc, f)
+    : HTMLGenericFormElementImpl(doc, f), m_options(0)
 {
     m_multiple = false;
     m_recalcListItems = false;
@@ -1945,6 +2046,10 @@ HTMLSelectElementImpl::HTMLSelectElementImpl(DocumentPtr *doc, HTMLFormElementIm
 HTMLSelectElementImpl::~HTMLSelectElementImpl()
 {
     if (getDocument()) getDocument()->deregisterMaintainsState(this);
+    if (m_options) {
+        m_options->detach();
+        m_options->deref();
+    }
 }
 
 NodeImpl::Id HTMLSelectElementImpl::id() const
@@ -2011,13 +2116,13 @@ long HTMLSelectElementImpl::length() const
     return len;
 }
 
-void HTMLSelectElementImpl::add( const HTMLElement &element, const HTMLElement &before )
+void HTMLSelectElementImpl::add( HTMLElementImpl *element, HTMLElementImpl *before )
 {
-    if(element.isNull() || element.handle()->id() != ID_OPTION)
+    if (!element || element->id() != ID_OPTION)
         return;
 
     int exceptioncode = 0;
-    insertBefore(element.handle(), before.handle(), exceptioncode );
+    insertBefore(element, before, exceptioncode);
     if (!exceptioncode)
         setRecalcListItems();
 }
@@ -2059,11 +2164,16 @@ DOMString HTMLSelectElementImpl::value( )
     return DOMString("");
 }
 
-void HTMLSelectElementImpl::setValue(DOMStringImpl* /*value*/)
+void HTMLSelectElementImpl::setValue(DOMStringImpl* value)
 {
-    // ### find the option with value() matching the given parameter
+    // find the option with value() matching the given parameter
     // and make it the current selection.
-    kdWarning() << "Unimplemented HTMLSelectElementImpl::setValue called" << endl;
+    QMemArray<HTMLGenericFormElementImpl*> items = listItems();
+    for (unsigned i = 0; i < items.size(); i++)
+        if (items[i]->id() == ID_OPTION && static_cast<HTMLOptionElementImpl*>(items[i])->value() == value) {
+            static_cast<HTMLOptionElementImpl*>(items[i])->setSelected(true);
+            return;
+        }
 }
 
 QString HTMLSelectElementImpl::state( )
@@ -2163,18 +2273,18 @@ NodeImpl* HTMLSelectElementImpl::addChild(NodeImpl* newChild)
     return HTMLGenericFormElementImpl::addChild(newChild);
 }
 
-void HTMLSelectElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLSelectElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
     case ATTR_SIZE:
-        m_size = QMAX( attr->val()->toInt(), 1 );
+        m_size = QMAX( attr->value().toInt(), 1 );
         break;
     case ATTR_WIDTH:
-        m_minwidth = QMAX( attr->val()->toInt(), 0 );
+        m_minwidth = QMAX( attr->value().toInt(), 0 );
         break;
     case ATTR_MULTIPLE:
-        m_multiple = (attr->val() != 0);
+        m_multiple = (!attr->isNull());
         break;
     case ATTR_ACCESSKEY:
         // ### ignore for the moment
@@ -2192,7 +2302,7 @@ void HTMLSelectElementImpl::parseAttribute(AttributeImpl *attr)
             getDocument()->createHTMLEventListener(attr->value().string()));
         break;
     default:
-        HTMLGenericFormElementImpl::parseAttribute(attr);
+        HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -2269,6 +2379,15 @@ int HTMLSelectElementImpl::listToOptionIndex(int listIndex) const
     return optionIndex;
 }
 
+HTMLOptionsCollectionImpl *HTMLSelectElementImpl::options()
+{
+    if (!m_options) {
+        m_options = new HTMLOptionsCollectionImpl(this);
+        m_options->ref();
+    }
+    return m_options;
+}
+
 void HTMLSelectElementImpl::recalcListItems()
 {
     NodeImpl* current = firstChild();
@@ -2402,19 +2521,19 @@ DOMString HTMLKeygenElementImpl::type() const
     return "keygen";
 }
 
-void HTMLKeygenElementImpl::parseAttribute(AttributeImpl* attr)
+void HTMLKeygenElementImpl::parseHTMLAttribute(HTMLAttributeImpl* attr)
 {
     switch(attr->id())
     {
     case ATTR_CHALLENGE:
-        m_challenge = attr->val();
+        m_challenge = attr->value();
         break;
     case ATTR_KEYTYPE:
-        m_keyType = attr->val();
+        m_keyType = attr->value();
         break;
     default:
         // skip HTMLSelectElementImpl parsing!
-        HTMLGenericFormElementImpl::parseAttribute(attr);
+        HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -2425,7 +2544,7 @@ bool HTMLKeygenElementImpl::encoding(const QTextCodec* codec, khtml::encodingLis
 
 #if APPLE_CHANGES
     // Only RSA is supported at this time.
-    if (!m_keyType.isNull() && m_keyType.lower() != "rsa") {
+    if (!m_keyType.isNull() && strcasecmp(m_keyType, "rsa")) {
         return false;
     }
     QString value = KSSLKeyGen::signedPublicKeyAndChallengeString((unsigned)selectedIndex(), m_challenge.string(), getDocument()->part()->baseURL());
@@ -2516,9 +2635,9 @@ NodeImpl* HTMLOptGroupElementImpl::addChild(NodeImpl* newChild)
     return HTMLGenericFormElementImpl::addChild(newChild);
 }
 
-void HTMLOptGroupElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLOptGroupElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
-    HTMLGenericFormElementImpl::parseAttribute(attr);
+    HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     recalcSelectOptions();
 }
 
@@ -2556,7 +2675,10 @@ DOMString HTMLOptionElementImpl::type() const
 
 DOMString HTMLOptionElementImpl::text() const
 {
-    DOMString label = getAttribute(ATTR_LABEL);
+    DOMString label;
+    // WinIE does not use the label attribute, so as a quirk, we ignore it.
+    if (getDocument() && !getDocument()->inCompatMode())
+        label = getAttribute(ATTR_LABEL);
     if (label.isEmpty() && firstChild() && firstChild()->nodeType() == Node::TEXT_NODE) {
 	if (firstChild()->nextSibling()) {
 	    DOMString ret = "";
@@ -2600,18 +2722,18 @@ void HTMLOptionElementImpl::setIndex( long  )
     // ###
 }
 
-void HTMLOptionElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLOptionElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
     case ATTR_SELECTED:
-        m_selected = (attr->val() != 0);
+        m_selected = (!attr->isNull());
         break;
     case ATTR_VALUE:
         m_value = attr->value();
         break;
     default:
-        HTMLGenericFormElementImpl::parseAttribute(attr);
+        HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -2701,15 +2823,19 @@ void HTMLTextAreaElementImpl::select(  )
     onSelect();
 }
 
-void HTMLTextAreaElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLTextAreaElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
     case ATTR_ROWS:
-        m_rows = attr->val() ? attr->val()->toInt() : 3;
+        m_rows = !attr->isNull() ? attr->value().toInt() : 3;
+        if (renderer())
+            renderer()->setNeedsLayoutAndMinMaxRecalc();
         break;
     case ATTR_COLS:
-        m_cols = attr->val() ? attr->val()->toInt() : 60;
+        m_cols = !attr->isNull() ? attr->value().toInt() : 60;
+        if (renderer())
+            renderer()->setNeedsLayoutAndMinMaxRecalc();
         break;
     case ATTR_WRAP:
         // virtual / physical is Netscape extension of HTML 3.0, now deprecated
@@ -2722,6 +2848,8 @@ void HTMLTextAreaElementImpl::parseAttribute(AttributeImpl *attr)
             m_wrap = ta_Physical;
         else if(strcasecmp( attr->value(), "off") == 0)
             m_wrap = ta_NoWrap;
+        if (renderer())
+            renderer()->setNeedsLayoutAndMinMaxRecalc();
         break;
     case ATTR_ACCESSKEY:
         // ignore for the moment
@@ -2743,7 +2871,7 @@ void HTMLTextAreaElementImpl::parseAttribute(AttributeImpl *attr)
 	    getDocument()->createHTMLEventListener(attr->value().string()));
         break;
     default:
-        HTMLGenericFormElementImpl::parseAttribute(attr);
+        HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -2857,27 +2985,45 @@ HTMLIsIndexElementImpl::HTMLIsIndexElementImpl(DocumentPtr *doc, HTMLFormElement
     setName("isindex");
 }
 
-HTMLIsIndexElementImpl::~HTMLIsIndexElementImpl()
-{
-}
-
 NodeImpl::Id HTMLIsIndexElementImpl::id() const
 {
     return ID_ISINDEX;
 }
 
-void HTMLIsIndexElementImpl::parseAttribute(AttributeImpl* attr)
+void HTMLIsIndexElementImpl::parseHTMLAttribute(HTMLAttributeImpl* attr)
 {
     switch(attr->id())
     {
     case ATTR_PROMPT:
 	setValue(attr->value());
     default:
-        // don't call HTMLInputElement::parseAttribute here, as it would
+        // don't call HTMLInputElement::parseHTMLAttribute here, as it would
         // accept attributes this element does not support
-        HTMLGenericFormElementImpl::parseAttribute(attr);
+        HTMLGenericFormElementImpl::parseHTMLAttribute(attr);
     }
 }
 
 // -------------------------------------------------------------------------
 
+unsigned long HTMLOptionsCollectionImpl::length() const
+{
+    // Not yet implemented.
+    return 0;
+}
+
+void HTMLOptionsCollectionImpl::setLength(unsigned long length)
+{
+    // Not yet implemented.
+}
+
+NodeImpl *HTMLOptionsCollectionImpl::item(unsigned long index) const
+{
+    // Not yet implemented.
+    return 0;
+}
+
+NodeImpl *HTMLOptionsCollectionImpl::namedItem(const DOMString &name) const
+{
+    // Not yet implemented.
+    return 0;
+}
