@@ -3,6 +3,7 @@
  *
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
+ * Copyright (C) 2004 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,7 +24,7 @@
 #ifndef HTML_OBJECTIMPL_H
 #define HTML_OBJECTIMPL_H
 
-#include "html_elementimpl.h"
+#include "html_imageimpl.h"
 #include "xml/dom_stringimpl.h"
 #include "java/kjavaappletcontext.h"
 
@@ -51,7 +52,9 @@ public:
 
     virtual Id id() const;
 
-    virtual void parseAttribute(AttributeImpl *token);
+    virtual bool mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const;
+    virtual void parseHTMLAttribute(HTMLAttributeImpl *token);
+    
     virtual bool rendererIsNeeded(khtml::RenderStyle *);
     virtual khtml::RenderObject *createRenderer(RenderArena *, khtml::RenderStyle *);
 
@@ -67,7 +70,9 @@ protected:
     khtml::VAlign valign;
 
 private:
+#if APPLE_CHANGES
     mutable KJS::Bindings::Instance *appletInstance;
+#endif
 };
 
 // -------------------------------------------------------------------------
@@ -81,16 +86,27 @@ public:
 
     virtual Id id() const;
 
-    virtual void parseAttribute(AttributeImpl *attr);
+    virtual bool mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const;
+    virtual void parseHTMLAttribute(HTMLAttributeImpl *attr);
 
     virtual void attach();
     virtual bool rendererIsNeeded(khtml::RenderStyle *);
     virtual khtml::RenderObject *createRenderer(RenderArena *, khtml::RenderStyle *);
+    
+    virtual bool isURLAttribute(AttributeImpl *attr) const;
+
+#if APPLE_CHANGES
+    KJS::Bindings::Instance *getEmbedInstance() const;
+#endif
 
     QString url;
     QString pluginPage;
     QString serviceType;
-    bool hidden;
+
+#if APPLE_CHANGES
+private:
+    mutable KJS::Bindings::Instance *embedInstance;
+#endif
 };
 
 // -------------------------------------------------------------------------
@@ -106,21 +122,25 @@ public:
 
     HTMLFormElementImpl *form() const;
 
-    virtual void parseAttribute(AttributeImpl *token);
+    virtual bool mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const;
+    virtual void parseHTMLAttribute(HTMLAttributeImpl *token);
 
     virtual void attach();
     virtual bool rendererIsNeeded(khtml::RenderStyle *);
     virtual khtml::RenderObject *createRenderer(RenderArena *, khtml::RenderStyle *);
     virtual void detach();
-
+    
     virtual void recalcStyle( StyleChange ch );
 
     DocumentImpl* contentDocument() const;
+    
+    virtual bool isURLAttribute(AttributeImpl *attr) const;
 
     QString serviceType;
     QString url;
     QString classId;
     bool needWidgetUpdate;
+    HTMLImageLoader* m_imageLoader;
 };
 
 // -------------------------------------------------------------------------
@@ -135,14 +155,16 @@ public:
 
     virtual Id id() const;
 
-    virtual void parseAttribute(AttributeImpl *token);
+    virtual void parseHTMLAttribute(HTMLAttributeImpl *token);
 
-    QString name() const { if(!m_name) return QString::null; return QString(m_name->s, m_name->l); }
-    QString value() const { if(!m_value) return QString::null; return QString(m_value->s, m_value->l); }
+    QString name() const { return m_name.string(); }
+    QString value() const { return m_value.string(); }
+    
+    virtual bool isURLAttribute(AttributeImpl *attr) const;
 
  protected:
-    DOMStringImpl *m_name;
-    DOMStringImpl *m_value;
+    AtomicString m_name;
+    AtomicString m_value;
 };
 
 };
