@@ -61,9 +61,7 @@ HTMLTableElementImpl::HTMLTableElementImpl(DocumentPtr *doc)
     rules = None;
     frame = Void;
 #endif
-
-    m_sharedCellDecls = 0;
-    
+ 
     padding = 1;
     
     m_noBorder = true;
@@ -72,11 +70,6 @@ HTMLTableElementImpl::HTMLTableElementImpl(DocumentPtr *doc)
 
 HTMLTableElementImpl::~HTMLTableElementImpl()
 {
-    if (m_sharedCellDecls) {
-        m_sharedCellDecls->setNode(0);
-        m_sharedCellDecls->setParent(0);
-        m_sharedCellDecls->deref();
-    }
 }
 
 NodeImpl::Id HTMLTableElementImpl::id() const
@@ -84,31 +77,6 @@ NodeImpl::Id HTMLTableElementImpl::id() const
     return ID_TABLE;
 }
 
-DOM::CSSStyleDeclarationImpl* HTMLTableElementImpl::createSharedCellDecls()
-{
-    if (!m_sharedCellDecls) {
-        m_sharedCellDecls = new CSSStyleDeclarationImpl(0);
-        m_sharedCellDecls->ref();
-        m_sharedCellDecls->setParent(getDocument()->elementSheet());
-        m_sharedCellDecls->setNode(this);
-        m_sharedCellDecls->setStrictParsing( !getDocument()->inCompatMode() );
-
-        if (m_noBorder)
-            m_sharedCellDecls->setProperty(CSS_PROP_BORDER_WIDTH, "0", false, true);
-        else {
-            m_sharedCellDecls->setProperty(CSS_PROP_BORDER_WIDTH, "1px", false, true);
-            int v = m_solid ? CSS_VAL_SOLID : CSS_VAL_INSET;
-            m_sharedCellDecls->setProperty(CSS_PROP_BORDER_TOP_STYLE, v, false, true);
-            m_sharedCellDecls->setProperty(CSS_PROP_BORDER_BOTTOM_STYLE, v, false, true);
-            m_sharedCellDecls->setProperty(CSS_PROP_BORDER_LEFT_STYLE, v, false, true);
-            m_sharedCellDecls->setProperty(CSS_PROP_BORDER_RIGHT_STYLE, v, false, true);
-            m_sharedCellDecls->setProperty(CSS_PROP_BORDER_COLOR, "inherit", false, true);
-        }
-    }
-    
-    return m_sharedCellDecls;
-}
-
 NodeImpl* HTMLTableElementImpl::setCaption( HTMLTableCaptionElementImpl *c )
 {
     int exceptioncode = 0;
@@ -348,24 +316,6 @@ NodeImpl *HTMLTableElementImpl::addChild(NodeImpl *child)
         return this;
     }
     
-    // We do not want this check-node-allowed test here, however the code to
-    // build up tables relies on childAllowed failing to make sure that the
-    // table is well-formed, the primary work being to add a tbody element.
-    // As 99.9999% of tables on the weeb do not have tbody elements, it seems
-    // odd to traverse an "error" case for the most common table markup.
-    // See <rdar://problem/3719373> Table parsing and construction relies on 
-    // childAllowed failures to build up proper DOM
-    if (child->nodeType() == Node::DOCUMENT_FRAGMENT_NODE) {
-        // child is a DocumentFragment... check all its children instead of child itself
-        for (NodeImpl *c = child->firstChild(); c; c = c->nextSibling())
-            if (!childAllowed(c))
-                return 0;
-    }
-    else if (!childAllowed(child)) {
-        // child is not a DocumentFragment... check if it's allowed directly
-        return 0;
-    }
-
     int exceptioncode = 0;
     NodeImpl *retval = appendChild( child, exceptioncode );
     if ( retval ) {
@@ -394,43 +344,73 @@ NodeImpl *HTMLTableElementImpl::addChild(NodeImpl *child)
     return retval;
 }
 
-void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
+bool HTMLTableElementImpl::mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const
+{
+    switch(attr) {
+        case ATTR_WIDTH:
+        case ATTR_HEIGHT:
+        case ATTR_BGCOLOR:
+        case ATTR_BACKGROUND:
+        case ATTR_CELLSPACING:
+        case ATTR_VSPACE:
+        case ATTR_HSPACE:
+        case ATTR_VALIGN:
+            result = eUniversal;
+            return false;
+        case ATTR_BORDERCOLOR:
+            result = eUniversal;
+            return true;
+        case ATTR_BORDER:
+            result = eTable;
+            return true;
+        case ATTR_ALIGN:
+            result = eTable;
+            return false;
+        default:
+            break;
+    }
+    return HTMLElementImpl::mapToEntry(attr, result);
+}
+
+void HTMLTableElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
-    // ### to CSS!!
     switch(attr->id())
     {
     case ATTR_WIDTH:
-        if (!attr->value().isEmpty())
-            addCSSLength( CSS_PROP_WIDTH, attr->value() );
-        else
-            removeCSSProperty(CSS_PROP_WIDTH);
+        addCSSLength(attr, CSS_PROP_WIDTH, attr->value());
         break;
     case ATTR_HEIGHT:
-        if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_HEIGHT, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_HEIGHT);
+        addCSSLength(attr, CSS_PROP_HEIGHT, attr->value());
         break;
     case ATTR_BORDER:
     {
-        int border;
-        // ### this needs more work, as the border value is not only
-        //     the border of the box, but also between the cells
-        if(!attr->val())
-            border = 0;
-        else if(attr->val()->l == 0)
-            border = 1;
-        else
-            border = attr->val()->toInt();
+        m_noBorder = true;
+        if (attr->isNull()) break;
+        if (attr->decl()) {
+            CSSValueImpl* val = attr->decl()->getPropertyCSSValue(CSS_PROP_BORDER_LEFT_WIDTH);
+            if (val && val->isPrimitiveValue()) {
+                CSSPrimitiveValueImpl* primVal = static_cast<CSSPrimitiveValueImpl*>(val);
+                m_noBorder = !primVal->getFloatValue(CSSPrimitiveValue::CSS_NUMBER);
+            }
+        }
+        else {
+            // ### this needs more work, as the border value is not only
+            //     the border of the box, but also between the cells
+            int border = 0;
+            if (attr->isEmpty())
+                border = 1;
+            else
+                border = attr->value().toInt();
 #ifdef DEBUG_DRAW_BORDER
-        border=1;
+            border=1;
 #endif
-        m_noBorder = !border;
-        DOMString v = QString::number( border );
-        addCSSLength(CSS_PROP_BORDER_WIDTH, v );
+            m_noBorder = !border;
+            DOMString v = QString::number( border );
+            addCSSLength(attr, CSS_PROP_BORDER_WIDTH, v );
+        }
 #if 0
         // wanted by HTML4 specs
-        if(!border)
+        if (m_noBorder)
             frame = Void, rules = None;
         else
             frame = Box, rules = All;
@@ -438,30 +418,24 @@ void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     }
     case ATTR_BGCOLOR:
-        if (!attr->value().isEmpty())
-            addHTMLColor(CSS_PROP_BACKGROUND_COLOR, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_BACKGROUND_COLOR);
+        addHTMLColor(attr, CSS_PROP_BACKGROUND_COLOR, attr->value());
         break;
     case ATTR_BORDERCOLOR:
-        if(!attr->value().isEmpty()) {
-            addHTMLColor(CSS_PROP_BORDER_COLOR, attr->value());
-            addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
+        m_solid = attr->decl();
+        if (!attr->decl() && !attr->isEmpty()) {
+            addHTMLColor(attr, CSS_PROP_BORDER_COLOR, attr->value());
+            addCSSProperty(attr, CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(attr, CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(attr, CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(attr, CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
             m_solid = true;
         }
         break;
     case ATTR_BACKGROUND:
     {
-        if (!attr->value().isEmpty()) {
-            QString url = khtml::parseURL( attr->value() ).string();
-            url = getDocument()->completeURL( url );
-            addCSSProperty(CSS_PROP_BACKGROUND_IMAGE, "url('"+url+"')" );
-        }
-        else
-            removeCSSProperty(CSS_PROP_BACKGROUND_IMAGE);
+        QString url = khtml::parseURL( attr->value() ).string();
+        if (!url.isEmpty())
+            addCSSImageProperty(attr, CSS_PROP_BACKGROUND_IMAGE, getDocument()->completeURL(url));
         break;
     }
     case ATTR_FRAME:
@@ -502,9 +476,7 @@ void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
         break;
    case ATTR_CELLSPACING:
         if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_BORDER_SPACING, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_BORDER_SPACING);
+            addCSSLength(attr, CSS_PROP_BORDER_SPACING, attr->value());
         break;
     case ATTR_CELLPADDING:
         if (!attr->value().isEmpty())
@@ -528,117 +500,176 @@ void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
 #endif
     }
     case ATTR_VSPACE:
-        addCSSLength(CSS_PROP_MARGIN_TOP, attr->value());
-        addCSSLength(CSS_PROP_MARGIN_BOTTOM, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_TOP, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_BOTTOM, attr->value());
         break;
     case ATTR_HSPACE:
-        addCSSLength(CSS_PROP_MARGIN_LEFT, attr->value());
-        addCSSLength(CSS_PROP_MARGIN_RIGHT, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_LEFT, attr->value());
+        addCSSLength(attr, CSS_PROP_MARGIN_RIGHT, attr->value());
         break;
     case ATTR_ALIGN:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_FLOAT, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_FLOAT);
+            addCSSProperty(attr, CSS_PROP_FLOAT, attr->value());
         break;
     case ATTR_VALIGN:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_VERTICAL_ALIGN);
+            addCSSProperty(attr, CSS_PROP_VERTICAL_ALIGN, attr->value());
         break;
     case ATTR_NOSAVE:
 	break;
     default:
-        HTMLElementImpl::parseAttribute(attr);
+        HTMLElementImpl::parseHTMLAttribute(attr);
     }
 }
 
+CSSStyleDeclarationImpl* HTMLTableElementImpl::additionalAttributeStyleDecl()
+{
+    if (m_noBorder)
+        return 0;
+    HTMLAttributeImpl attr(ATTR_TABLEBORDER, m_solid ? "solid" : "outset");
+    CSSMappedAttributeDeclarationImpl* decl = getMappedAttributeDecl(ePersistent, &attr);
+    if (!decl) {
+        decl = new CSSMappedAttributeDeclarationImpl(0);
+        decl->setParent(getDocument()->elementSheet());
+        decl->setNode(this);
+        decl->setStrictParsing(false); // Mapped attributes are just always quirky.
+        
+        decl->ref(); // This single ref pins us in the table until the document dies.
+
+        int v = m_solid ? CSS_VAL_SOLID : CSS_VAL_OUTSET;
+        decl->setProperty(CSS_PROP_BORDER_TOP_STYLE, v, false);
+        decl->setProperty(CSS_PROP_BORDER_BOTTOM_STYLE, v, false);
+        decl->setProperty(CSS_PROP_BORDER_LEFT_STYLE, v, false);
+        decl->setProperty(CSS_PROP_BORDER_RIGHT_STYLE, v, false);
+
+        setMappedAttributeDecl(ePersistent, &attr, decl);
+        decl->setParent(0);
+        decl->setNode(0);
+        decl->setMappedState(ePersistent, attr.id(), attr.value());
+    }
+    return decl;
+}
+
+CSSStyleDeclarationImpl* HTMLTableElementImpl::getSharedCellDecl()
+{
+    HTMLAttributeImpl attr(ATTR_CELLBORDER, m_noBorder ? "none" : (m_solid ? "solid" : "inset"));
+    CSSMappedAttributeDeclarationImpl* decl = getMappedAttributeDecl(ePersistent, &attr);
+    if (!decl) {
+        decl = new CSSMappedAttributeDeclarationImpl(0);
+        decl->setParent(getDocument()->elementSheet());
+        decl->setNode(this);
+        decl->setStrictParsing(false); // Mapped attributes are just always quirky.
+        
+        decl->ref(); // This single ref pins us in the table until the table dies.
+        
+        if (m_noBorder)
+            decl->setProperty(CSS_PROP_BORDER_WIDTH, "0", false);
+        else {
+            decl->setProperty(CSS_PROP_BORDER_WIDTH, "1px", false);
+            int v = m_solid ? CSS_VAL_SOLID : CSS_VAL_INSET;
+            decl->setProperty(CSS_PROP_BORDER_TOP_STYLE, v, false);
+            decl->setProperty(CSS_PROP_BORDER_BOTTOM_STYLE, v, false);
+            decl->setProperty(CSS_PROP_BORDER_LEFT_STYLE, v, false);
+            decl->setProperty(CSS_PROP_BORDER_RIGHT_STYLE, v, false);
+            decl->setProperty(CSS_PROP_BORDER_COLOR, "inherit", false);
+        }
+
+        setMappedAttributeDecl(ePersistent, &attr, decl);
+        decl->setParent(0);
+        decl->setNode(0);
+        decl->setMappedState(ePersistent, attr.id(), attr.value());
+    }
+    return decl;
+}
+
 void HTMLTableElementImpl::attach()
 {
     assert(!m_attached);
-    if (parent()->renderer()) {
-        if (!m_noBorder) {
-            int v = m_solid ? CSS_VAL_SOLID : CSS_VAL_OUTSET;
-            addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, v);
-            addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, v);
-            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, v);
-            addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, v);
-        }
-    }
-
     HTMLElementImpl::attach();
     if ( m_render && m_render->isTable() )
 	static_cast<RenderTable *>(m_render)->setCellPadding( padding );
 }
 
+bool HTMLTableElementImpl::isURLAttribute(AttributeImpl *attr) const
+{
+    return attr->id() == ATTR_BACKGROUND;
+}
+
 // --------------------------------------------------------------------------
 
-void HTMLTablePartElementImpl::parseAttribute(AttributeImpl *attr)
+bool HTMLTablePartElementImpl::mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const
+{
+    switch(attr) {
+        case ATTR_BGCOLOR:
+        case ATTR_BACKGROUND:
+        case ATTR_BORDERCOLOR:
+        case ATTR_VALIGN:
+        case ATTR_HEIGHT:
+            result = eUniversal;
+            return false;
+        case ATTR_ALIGN:
+            result = eCell; // All table parts will just share in the TD space.
+            return false;
+        default:
+            break;
+    }
+    return HTMLElementImpl::mapToEntry(attr, result);
+}
+
+void HTMLTablePartElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
     case ATTR_BGCOLOR:
-        if (attr->val())
-            addHTMLColor(CSS_PROP_BACKGROUND_COLOR, attr->value() );
-        else
-            removeCSSProperty(CSS_PROP_BACKGROUND_COLOR);
+        addHTMLColor(attr, CSS_PROP_BACKGROUND_COLOR, attr->value() );
         break;
     case ATTR_BACKGROUND:
     {
-        if (attr->val()) {
-            QString url = khtml::parseURL( attr->value() ).string();
-            url = getDocument()->completeURL( url );
-            addCSSProperty(CSS_PROP_BACKGROUND_IMAGE,  "url('"+url+"')" );
-        }
-        else
-            removeCSSProperty(CSS_PROP_BACKGROUND_IMAGE);
+        QString url = khtml::parseURL( attr->value() ).string();
+        if (!url.isEmpty())
+            addCSSImageProperty(attr, CSS_PROP_BACKGROUND_IMAGE, getDocument()->completeURL(url));
         break;
     }
     case ATTR_BORDERCOLOR:
     {
-        if(!attr->value().isEmpty()) {
-            addHTMLColor(CSS_PROP_BORDER_COLOR, attr->value());
-            addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
-            addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
+        if (!attr->value().isEmpty()) {
+            addHTMLColor(attr, CSS_PROP_BORDER_COLOR, attr->value());
+            addCSSProperty(attr, CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(attr, CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(attr, CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
+            addCSSProperty(attr, CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
         }
         break;
     }
     case ATTR_VALIGN:
     {
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_VERTICAL_ALIGN);
+            addCSSProperty(attr, CSS_PROP_VERTICAL_ALIGN, attr->value());
         break;
     }
     case ATTR_ALIGN:
     {
         DOMString v = attr->value();
         if ( strcasecmp( attr->value(), "middle" ) == 0 || strcasecmp( attr->value(), "center" ) == 0 )
-            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_CENTER);
+            addCSSProperty(attr, CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_CENTER);
         else if (strcasecmp(attr->value(), "absmiddle") == 0)
-            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL_CENTER);
+            addCSSProperty(attr, CSS_PROP_TEXT_ALIGN, CSS_VAL_CENTER);
         else if (strcasecmp(attr->value(), "left") == 0)
-            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_LEFT);
+            addCSSProperty(attr, CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_LEFT);
         else if (strcasecmp(attr->value(), "right") == 0)
-            addCSSProperty(CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_RIGHT);
+            addCSSProperty(attr, CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_RIGHT);
         else
-            addCSSProperty(CSS_PROP_TEXT_ALIGN, v);
+            addCSSProperty(attr, CSS_PROP_TEXT_ALIGN, v);
         break;
     }
     case ATTR_HEIGHT:
         if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_HEIGHT, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_HEIGHT);
+            addCSSLength(attr, CSS_PROP_HEIGHT, attr->value());
         break;
     case ATTR_NOSAVE:
 	break;
     default:
-        HTMLElementImpl::parseAttribute(attr);
+        HTMLElementImpl::parseHTMLAttribute(attr);
     }
 }
 
@@ -860,59 +891,66 @@ HTMLTableCellElementImpl::~HTMLTableCellElementImpl()
 {
 }
 
-void HTMLTableCellElementImpl::parseAttribute(AttributeImpl *attr)
+
+bool HTMLTableCellElementImpl::mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const
+{
+    switch(attr) {
+        case ATTR_NOWRAP:
+            result = eUniversal;
+            return false;
+        case ATTR_WIDTH:
+        case ATTR_HEIGHT:
+            result = eCell; // Because of the quirky behavior of ignoring 0 values, cells are special.
+            return false;
+        default:
+            break;
+    }
+    return HTMLTablePartElementImpl::mapToEntry(attr, result);
+}
+
+void HTMLTableCellElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
-    case ATTR_BORDER:
-        // euhm? not supported by other browsers as far as I can see (Dirk)
-        //addCSSLength(CSS_PROP_BORDER_WIDTH, attr->value());
-        break;
     case ATTR_ROWSPAN:
         // ###
-        rSpan = attr->val() ? attr->val()->toInt() : 1;
+        rSpan = !attr->isNull() ? attr->value().toInt() : 1;
         // limit this to something not causing an overflow with short int
         if(rSpan < 1 || rSpan > 1024) rSpan = 1;
         break;
     case ATTR_COLSPAN:
         // ###
-        cSpan = attr->val() ? attr->val()->toInt() : 1;
+        cSpan = !attr->isNull() ? attr->value().toInt() : 1;
         // limit this to something not causing an overflow with short int
         if(cSpan < 1 || cSpan > 1024) cSpan = 1;
         break;
     case ATTR_NOWRAP:
-        if (attr->val() != 0)
-            addCSSProperty(CSS_PROP_WHITE_SPACE, CSS_VAL__KHTML_NOWRAP);
-        else
-            removeCSSProperty(CSS_PROP_WHITE_SPACE);
+        if (!attr->isNull())
+            addCSSProperty(attr, CSS_PROP_WHITE_SPACE, CSS_VAL__KHTML_NOWRAP);
         break;
     case ATTR_WIDTH:
         if (!attr->value().isEmpty()) {
-            int widthInt = attr->val()->toInt();
+            int widthInt = attr->value().toInt();
             if (widthInt > 0) // width="0" is ignored for compatibility with WinIE.
-                addCSSLength( CSS_PROP_WIDTH, attr->value() );
+                addCSSLength( attr, CSS_PROP_WIDTH, attr->value() );
         }
-        else
-            removeCSSProperty(CSS_PROP_WIDTH);
         break;
     case ATTR_HEIGHT:
         if (!attr->value().isEmpty()) {
-            int heightInt = attr->val()->toInt();
+            int heightInt = attr->value().toInt();
             if (heightInt > 0) // height="0" is ignored for compatibility with WinIE.
-                addCSSLength( CSS_PROP_HEIGHT, attr->value() );
+                addCSSLength( attr, CSS_PROP_HEIGHT, attr->value() );
         }
-        else
-            removeCSSProperty(CSS_PROP_HEIGHT);
         break;
     case ATTR_NOSAVE:
 	break;
     default:
-        HTMLTablePartElementImpl::parseAttribute(attr);
+        HTMLTablePartElementImpl::parseHTMLAttribute(attr);
     }
 }
 
 // used by table cells to share style decls created by the enclosing table.
-DOM::CSSStyleDeclarationImpl* HTMLTableCellElementImpl::getAdditionalStyleDecls()
+CSSStyleDeclarationImpl* HTMLTableCellElementImpl::additionalAttributeStyleDecl()
 {
     HTMLElementImpl* p = static_cast<HTMLElementImpl*>(parentNode());
     while(p && p->id() != ID_TABLE)
@@ -920,7 +958,7 @@ DOM::CSSStyleDeclarationImpl* HTMLTableCellElementImpl::getAdditionalStyleDecls(
 
     if (p) {
         HTMLTableElementImpl* table = static_cast<HTMLTableElementImpl*>(p);
-        return table->m_sharedCellDecls;
+        return table->getSharedCellDecl();
     }
 
     return 0;
@@ -932,14 +970,14 @@ void HTMLTableCellElementImpl::attach()
     while(p && p->id() != ID_TABLE)
         p = static_cast<HTMLElementImpl*>(p->parentNode());
 
-    if (p) {
-        HTMLTableElementImpl* table = static_cast<HTMLTableElementImpl*>(p);
-        table->createSharedCellDecls();
-    }
-
     HTMLTablePartElementImpl::attach();
 }
 
+bool HTMLTableCellElementImpl::isURLAttribute(AttributeImpl *attr) const
+{
+    return attr->id() == ATTR_BACKGROUND;
+}
+
 // -------------------------------------------------------------------------
 
 HTMLTableColElementImpl::HTMLTableColElementImpl(DocumentPtr *doc, ushort i)
@@ -954,28 +992,31 @@ NodeImpl::Id HTMLTableColElementImpl::id() const
     return _id;
 }
 
+bool HTMLTableColElementImpl::mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const
+{
+    switch(attr) {
+        case ATTR_WIDTH:
+            result = eUniversal;
+            return false;
+        default:
+            break;
+    }
+    return HTMLTablePartElementImpl::mapToEntry(attr, result);
+}
 
-void HTMLTableColElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLTableColElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
     case ATTR_SPAN:
-        _span = attr->val() ? attr->val()->toInt() : 1;
+        _span = !attr->isNull() ? attr->value().toInt() : 1;
         break;
     case ATTR_WIDTH:
         if (!attr->value().isEmpty())
-            addCSSLength(CSS_PROP_WIDTH, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_WIDTH);
-        break;
-    case ATTR_VALIGN:
-        if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_VERTICAL_ALIGN, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_VERTICAL_ALIGN);
+            addCSSLength(attr, CSS_PROP_WIDTH, attr->value());
         break;
     default:
-        HTMLTablePartElementImpl::parseAttribute(attr);
+        HTMLTablePartElementImpl::parseHTMLAttribute(attr);
     }
 
 }
@@ -987,19 +1028,25 @@ NodeImpl::Id HTMLTableCaptionElementImpl::id() const
     return ID_CAPTION;
 }
 
+bool HTMLTableCaptionElementImpl::mapToEntry(NodeImpl::Id attr, MappedAttributeEntry& result) const
+{
+    if (attr == ATTR_ALIGN) {
+        result = eCaption;
+        return false;
+    }
+    return HTMLElementImpl::mapToEntry(attr, result);
+}
 
-void HTMLTableCaptionElementImpl::parseAttribute(AttributeImpl *attr)
+void HTMLTableCaptionElementImpl::parseHTMLAttribute(HTMLAttributeImpl *attr)
 {
     switch(attr->id())
     {
     case ATTR_ALIGN:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_CAPTION_SIDE, attr->value());
-        else
-            removeCSSProperty(CSS_PROP_CAPTION_SIDE);
+            addCSSProperty(attr, CSS_PROP_CAPTION_SIDE, attr->value());
         break;
     default:
-        HTMLElementImpl::parseAttribute(attr);
+        HTMLElementImpl::parseHTMLAttribute(attr);
     }
 
 }
