@@ -101,7 +101,10 @@ namespace khtml
 	enum Type {
 	    Image,
 	    CSSStyleSheet,
-	    Script
+	    Script,
+#ifndef KHTML_NO_XBL
+            XBL
+#endif
 	};
 
 	enum Status {
@@ -359,6 +362,31 @@ namespace khtml
 #endif
     };
 
+#ifndef KHTML_NO_XBL
+    class CachedXBLDocument : public CachedObject
+    {
+    public:
+        CachedXBLDocument(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy, time_t _expireDate);
+        virtual ~CachedXBLDocument();
+        
+        XBL::XBLDocumentImpl* document() const { return m_document; }
+        
+        virtual void ref(CachedObjectClient *consumer);
+        virtual void deref(CachedObjectClient *consumer);
+        
+        virtual void data( QBuffer &buffer, bool eof );
+        virtual void error( int err, const char *text );
+        
+        virtual bool schedule() const { return true; }
+        
+        void checkNotify();
+        
+protected:
+        XBL::XBLDocumentImpl* m_document;
+        QTextCodec* m_codec;
+    };
+#endif
+
     /**
      * @internal
      *
@@ -374,6 +402,10 @@ namespace khtml
 	CachedCSSStyleSheet *requestStyleSheet( const DOM::DOMString &url, const QString& charset);
         CachedScript *requestScript( const DOM::DOMString &url, const QString& charset);
 
+#ifndef KHTML_NO_XBL
+        CachedXBLDocument* requestXBLDocument(const DOM::DOMString &url);
+#endif
+
 	bool autoloadImages() const { return m_bautoloadImages; }
         KIO::CacheControl cachePolicy() const { return m_cachePolicy; }
         KHTMLSettings::KAnimationAdvice showAnimations() const { return m_showAnimations; }
@@ -487,6 +519,7 @@ namespace khtml
          * Otherwise, it is automatically base-url expanded
 	 */
 	static CachedImage *requestImage( DocLoader* l, const DOM::DOMString &url, bool reload=false, time_t _expireDate=0);
+        static CachedImage *requestImage( DocLoader* l, const KURL &url, bool reload=false, time_t _expireDate=0);
 
 	/**
 	 * Ask the cache for some url. Will return a cachedObject, and
@@ -494,6 +527,12 @@ namespace khtml
 	 */
 	static CachedCSSStyleSheet *requestStyleSheet( DocLoader* l, const DOM::DOMString &url, bool reload=false, time_t _expireDate=0, const QString& charset = QString::null);
 
+#ifndef KHTML_NO_XBL
+        // Ask the cache for an XBL document.
+        static CachedXBLDocument* requestXBLDocument(DocLoader* l, const DOM::DOMString &url, 
+                                                     bool reload=false, time_t _expireDate=0);
+#endif
+
         /**
          * Pre-loads a stylesheet into the cache.
          */
@@ -556,6 +595,9 @@ namespace khtml
             TypeStatistic movies;
             TypeStatistic styleSheets;
             TypeStatistic scripts;
+#ifndef KHTML_NO_XBL
+            TypeStatistic xblDocs;
+#endif
             TypeStatistic other;
         };
 
