@@ -31,6 +31,7 @@
 #include "render_image.h"
 #include "render_canvas.h"
 #include "xml/dom_docimpl.h"
+#include "xml/dom_position.h"
 
 #include <kdebug.h>
 #include <assert.h>
@@ -40,6 +41,7 @@
 #include "KWQAccObjectCache.h" 
 #endif
 
+using DOM::Position;
 using namespace khtml;
 
 RenderContainer::RenderContainer(DOM::NodeImpl* node)
@@ -143,7 +145,7 @@ void RenderContainer::addChild(RenderObject *newChild, RenderObject *beforeChild
         else {
             //kdDebug( 6040 ) << "creating anonymous table" << endl;
             table = new (renderArena()) RenderTable(document() /* is anonymous */);
-            RenderStyle *newStyle = new RenderStyle();
+            RenderStyle *newStyle = new (renderArena()) RenderStyle();
             newStyle->inheritFrom(style());
             newStyle->setDisplay(TABLE);
             table->setStyle(newStyle);
@@ -163,11 +165,9 @@ RenderObject* RenderContainer::removeChildNode(RenderObject* oldChild)
     // So that we'll get the appropriate dirty bit set (either that a normal flow child got yanked or
     // that a positioned child got yanked).  We also repaint, so that the area exposed when the child
     // disappears gets repainted properly.
-    if (document()->renderer()) {
+    if (!documentBeingDestroyed()) {
         oldChild->setNeedsLayoutAndMinMaxRecalc();
-#ifdef INCREMENTAL_REPAINTING
         oldChild->repaint();
-#endif
         
         // Keep our layer hierarchy updated.
         oldChild->removeLayers(enclosingLayer());
@@ -284,7 +284,7 @@ void RenderContainer::updatePseudoChild(RenderStyle::PseudoId type, RenderObject
                     genChild->setStyle(pseudo);
                 else {
                     // Images get an empty style that inherits from the pseudo.
-                    RenderStyle* style = new RenderStyle();
+                    RenderStyle* style = new (renderArena()) RenderStyle();
                     style->inheritFrom(pseudo);
                     genChild->setStyle(style);
                 }
@@ -315,7 +315,7 @@ void RenderContainer::updatePseudoChild(RenderStyle::PseudoId type, RenderObject
         else if (contentData->contentType() == CONTENT_OBJECT)
         {
             RenderImage* img = new (renderArena()) RenderImage(document()); /* Anonymous object */
-            RenderStyle* style = new RenderStyle();
+            RenderStyle* style = new (renderArena()) RenderStyle();
             style->inheritFrom(pseudo);
             img->setStyle(style);
             img->setContentObject(contentData->contentObject());
@@ -360,6 +360,9 @@ void RenderContainer::appendChildNode(RenderObject* newChild)
     if (!normalChildNeedsLayout())
         setChildNeedsLayout(true); // We may supply the static position for an absolute positioned child.
     
+    if (!newChild->isFloatingOrPositioned() && childrenInline())
+        dirtyLinesFromChangedChild(newChild);
+    
 #if APPLE_CHANGES
     KWQAccObjectCache* cache = document()->getExistingAccObjectCache();
     if (cache)
@@ -398,6 +401,9 @@ void RenderContainer::insertChildNode(RenderObject* child, RenderObject* beforeC
     if (!normalChildNeedsLayout())
         setChildNeedsLayout(true); // We may supply the static position for an absolute positioned child.
     
+    if (!child->isFloatingOrPositioned() && childrenInline())
+        dirtyLinesFromChangedChild(child);
+    
 #if APPLE_CHANGES
     KWQAccObjectCache* cache = document()->getExistingAccObjectCache();
     if (cache)
@@ -468,5 +474,33 @@ void RenderContainer::removeLeftoverAnonymousBoxes()
     if ( parent() )
 	parent()->removeLeftoverAnonymousBoxes();
 }
+
+Position RenderContainer::positionForCoordinates(int _x, int _y)
+{
+    // no children...return this render object's element, if there is one, and offset 0
+    if (!firstChild())
+        return Position(element(), 0);
+
+    // look for the geometrically-closest child and pass off to that child
+    int min = INT_MAX;
+    RenderObject *closestRenderer = firstChild();
+    for (RenderObject *renderer = firstChild(); renderer; renderer = renderer->nextSibling()) {
+        int absx, absy;
+        renderer->absolutePosition(absx, absy);
+        
+        int top = absy + borderTop() + paddingTop();
+        int bottom = top + renderer->contentHeight();
+        int left = absx + borderLeft() + paddingLeft();
+        int right = left + renderer->contentWidth();
+        
+        int cmp;
+        cmp = abs(_y - top);    if (cmp < min) { closestRenderer = renderer; min = cmp; }
+        cmp = abs(_y - bottom); if (cmp < min) { closestRenderer = renderer; min = cmp; }
+        cmp = abs(_x - left);   if (cmp < min) { closestRenderer = renderer; min = cmp; }
+        cmp = abs(_x - right);  if (cmp < min) { closestRenderer = renderer; min = cmp; }
+    }
+
+    return closestRenderer->positionForCoordinates(_x, _y);
+}
     
 #undef DEBUG_LAYOUT
