@@ -53,9 +53,6 @@
 class QScrollBar;
 template <class T> class QPtrVector;
 
-// Uncomment to enable incremental painting
-#define INCREMENTAL_REPAINTING 1
-
 namespace khtml {
     class RenderStyle;
     class RenderTable;
@@ -147,6 +144,8 @@ public:
     void removeOnlyThisLayer();
     void insertOnlyThisLayer();
 
+    void repaintIncludingDescendants();
+    
     void styleChanged();
     
     Marquee* marquee() const { return m_marquee; }
@@ -166,13 +165,13 @@ public:
     
     int xPos() const { return m_x; }
     int yPos() const { return m_y; }
-    short width() const { return m_width; }
+    int width() const { return m_width; }
     int height() const { return m_height; }
 
-    void setWidth(short w) { m_width = w; }
+    void setWidth(int w) { m_width = w; }
     void setHeight(int h) { m_height = h; }
 
-    short scrollWidth();
+    int scrollWidth();
     int scrollHeight();
     
     void setPos( int xPos, int yPos ) {
@@ -183,7 +182,7 @@ public:
     // Scrolling methods for layers that can scroll their overflow.
     void scrollOffset(int& x, int& y);
     void subtractScrollOffset(int& x, int& y);
-    short scrollXOffset() { return m_scrollX; }
+    int scrollXOffset() { return m_scrollX; }
     int scrollYOffset() { return m_scrollY; }
     void scrollToOffset(int x, int y, bool updateScrollbars = true, bool repaint = true);
     void scrollToXOffset(int x) { scrollToOffset(x, m_scrollY); }
@@ -204,15 +203,11 @@ public:
     void updateScrollPositionFromScrollbars();
 
     void updateLayerPosition();
-#ifdef INCREMENTAL_REPAINTING
     void updateLayerPositions(bool doFullRepaint = false, bool checkForRepaint=true);
     void computeRepaintRects();
     void relativePositionOffset(int& relX, int& relY) {
         relX += m_relX; relY += m_relY;
     }
-#else
-    void updateLayerPositions();
-#endif
 
     // Get the enclosing stacking context for this layer.  A stacking context is a layer
     // that has a non-auto z-index.
@@ -237,7 +232,7 @@ public:
     // paints the layers that intersect the damage rect from back to
     // front.  The nodeAtPoint method looks for mouse events by walking
     // layers that intersect the point from front to back.
-    void paint(QPainter *p, const QRect& damageRect, bool selectionOnly=false);
+    void paint(QPainter *p, const QRect& damageRect, bool selectionOnly=false, RenderObject *paintingRoot=0);
     bool nodeAtPoint(RenderObject::NodeInfo& info, int x, int y);
 
     // This method figures out our layerBounds in coordinates relative to
@@ -253,6 +248,8 @@ public:
     
     void updateHoverActiveState(RenderObject::NodeInfo& info);
     
+    QRect repaintRect() const { return m_repaintRect; }
+
     void detach(RenderArena* renderArena);
 
      // Overloaded new operator.  Derived classes must override operator new
@@ -276,7 +273,7 @@ private:
     void collectLayers(QPtrVector<RenderLayer>*&, QPtrVector<RenderLayer>*&);
 
     void paintLayer(RenderLayer* rootLayer, QPainter *p, const QRect& paintDirtyRect, 
-                    bool haveTransparency=false, bool selectionOnly=false);
+                    bool haveTransparency, bool selectionOnly, RenderObject *paintingRoot);
     RenderLayer* nodeAtPointForLayer(RenderLayer* rootLayer, RenderObject::NodeInfo& info,
                                      int x, int y, const QRect& hitTestRect);
 
@@ -292,29 +289,27 @@ protected:
     RenderLayer* m_first;
     RenderLayer* m_last;
 
-#ifdef INCREMENTAL_REPAINTING
     QRect m_repaintRect; // Cached repaint rects. Used by layout.
     QRect m_fullRepaintRect;
 
     // Our current relative position offset.
     int m_relX;
     int m_relY;
-#endif
 
     // Our (x,y) coordinates are in our parent layer's coordinate space.
-    short m_x;
+    int m_x;
     int m_y;
 
     // The layer's width/height
-    short m_width;
+    int m_width;
     int m_height;
     
     // Our scroll offsets if the view is scrolled.
-    short m_scrollX;
+    int m_scrollX;
     int m_scrollY;
     
     // The width/height of our scrolled area.
-    short m_scrollWidth;
+    int m_scrollWidth;
     int m_scrollHeight;
     
     // For layers with overflow, we have a pair of scrollbars.
