@@ -40,12 +40,44 @@
 #include "htmltags.h"
 
 using namespace DOM;
-using namespace khtml;
 
 #ifndef NDEBUG
 static bool inInlineBoxDetach;
 #endif
 
+namespace khtml {
+    
+class EllipsisBox : public InlineBox
+{
+public:
+    EllipsisBox(RenderObject* obj, const DOM::AtomicString& ellipsisStr, InlineFlowBox* p,
+                int w, int y, int h, int b, bool firstLine, InlineBox* markupBox)
+    :InlineBox(obj), m_str(ellipsisStr) {
+        m_parent = p;
+        m_width = w;
+        m_y = y;
+        m_height = h;
+        m_baseline = b;
+        m_firstLine = firstLine;
+        m_constructed = true;
+        m_markupBox = markupBox;
+    }
+    
+    void paint(RenderObject::PaintInfo& i, int _tx, int _ty);
+    bool nodeAtPoint(RenderObject::NodeInfo& info, int _x, int _y, int _tx, int _ty,
+                     HitTestAction hitTestAction, bool inBox);
+
+private:
+    DOM::AtomicString m_str;
+    InlineBox* m_markupBox;
+};
+
+void InlineBox::remove()
+{ 
+    if (parent())
+        parent()->removeChild(this);
+}
+
 void InlineBox::detach(RenderArena* renderArena)
 {
 #ifndef NDEBUG
@@ -74,6 +106,125 @@ void InlineBox::operator delete(void* ptr, size_t sz)
     *(size_t *)ptr = sz;
 }
 
+long InlineBox::caretMinOffset() const 
+{ 
+    return 0; 
+}
+
+long InlineBox::caretMaxOffset() const 
+{ 
+    return 1; 
+}
+
+unsigned long InlineBox::caretMaxRenderedOffset() const 
+{ 
+    return 1; 
+}
+
+void InlineBox::dirtyLineBoxes()
+{
+    markDirty();
+    for (InlineFlowBox* curr = parent(); curr && !curr->isDirty(); curr = curr->parent())
+        curr->markDirty();
+}
+
+void InlineBox::deleteLine(RenderArena* arena)
+{
+    detach(arena);
+}
+
+void InlineBox::extractLine()
+{
+    m_extracted = true;
+    m_object->setInlineBoxWrapper(0);
+}
+
+void InlineBox::attachLine()
+{
+    m_extracted = false;
+    m_object->setInlineBoxWrapper(this);
+}
+
+void InlineBox::adjustPosition(int dx, int dy)
+{
+    m_x += dx;
+    m_y += dy;
+    if (m_object->isReplaced() || m_object->isBR())
+        m_object->setPos(m_object->xPos() + dx, m_object->yPos() + dy);
+}
+
+RootInlineBox* InlineBox::root()
+{ 
+    if (m_parent)
+        return m_parent->root(); 
+    return static_cast<RootInlineBox*>(this);
+}
+
+bool InlineBox::nextOnLineExists() const
+{
+    if (!parent())
+        return false;
+    
+    if (nextOnLine())
+        return true;
+    
+    return parent()->nextOnLineExists();
+}
+
+bool InlineBox::prevOnLineExists() const
+{
+    if (!parent())
+        return false;
+    
+    if (prevOnLine())
+        return true;
+    
+    return parent()->prevOnLineExists();
+}
+
+InlineBox* InlineBox::firstLeafChild()
+{
+    return this;
+}
+
+InlineBox* InlineBox::lastLeafChild()
+{
+    return this;
+}
+
+InlineBox* InlineBox::closestLeafChildForXPos(int _x, int _tx)
+{
+    if (!isInlineFlowBox())
+        return this;
+    
+    InlineFlowBox *flowBox = static_cast<InlineFlowBox*>(this);
+    if (!flowBox->firstChild())
+        return this;
+
+    InlineBox *box = flowBox->closestChildForXPos(_x, _tx);
+    if (!box)
+        return this;
+    
+    return box->closestLeafChildForXPos(_x, _tx);
+}
+
+bool InlineBox::canAccommodateEllipsis(bool ltr, int blockEdge, int ellipsisWidth)
+{
+    // Non-replaced elements can always accommodate an ellipsis.
+    if (!m_object || !m_object->isReplaced())
+        return true;
+    
+    QRect boxRect(m_x, 0, m_width, 10);
+    QRect ellipsisRect(ltr ? blockEdge - ellipsisWidth : blockEdge, 0, ellipsisWidth, 10);
+    return !(boxRect.intersects(ellipsisRect));
+}
+
+int InlineBox::placeEllipsisBox(bool ltr, int blockEdge, int ellipsisWidth, bool&)
+{
+    // Use -1 to mean "we didn't set the position."
+    return -1;
+}
+
 int InlineFlowBox::marginLeft()
 {
     if (!includeLeftEdge())
@@ -118,26 +269,60 @@ int InlineFlowBox::getFlowSpacingWidth()
     return totWidth;
 }
 
-bool InlineFlowBox::nextOnLineExists()
+void InlineFlowBox::removeChild(InlineBox* child)
 {
-    if (!parent())
-        return false;
+    if (!m_dirty)
+        dirtyLineBoxes();
 
-    if (nextOnLine())
-        return true;
+    root()->childRemoved(child);
 
-    return parent()->nextOnLineExists();
+    if (child == m_firstChild)
+        m_firstChild = child->nextOnLine();
+    if (child == m_lastChild)
+        m_lastChild = child->prevOnLine();
+    if (child->nextOnLine())
+        child->nextOnLine()->setPrevOnLine(child->prevOnLine());
+    if (child->prevOnLine())
+        child->prevOnLine()->setNextOnLine(child->nextOnLine());
+    
+    child->setParent(0);
 }
 
-bool InlineFlowBox::prevOnLineExists()
+void InlineFlowBox::deleteLine(RenderArena* arena)
 {
-    if (!parent())
-        return false;
+    InlineBox* child = m_firstChild;
+    InlineBox* next = 0;
+    while (child) {
+        next = child->nextOnLine();
+        child->deleteLine(arena);
+        child = next;
+    }
+    
+    static_cast<RenderFlow*>(m_object)->removeLineBox(this);
+    detach(arena);
+}
 
-    if (prevOnLine())
-        return true;
+void InlineFlowBox::extractLine()
+{
+    if (!m_extracted)
+        static_cast<RenderFlow*>(m_object)->extractLineBox(this);
+    for (InlineBox* child = m_firstChild; child; child = child->nextOnLine())
+        child->extractLine();
+}
 
-    return parent()->prevOnLineExists();
+void InlineFlowBox::attachLine()
+{
+    if (m_extracted)
+        static_cast<RenderFlow*>(m_object)->attachLineBox(this);
+    for (InlineBox* child = m_firstChild; child; child = child->nextOnLine())
+        child->attachLine();
+}
+
+void InlineFlowBox::adjustPosition(int dx, int dy)
+{
+    InlineRunBox::adjustPosition(dx, dy);
+    for (InlineBox* child = m_firstChild; child; child = child->nextOnLine())
+        child->adjustPosition(dx, dy);
 }
 
 bool InlineFlowBox::onEndChain(RenderObject* endObject)
@@ -288,7 +473,7 @@ void InlineFlowBox::verticallyAlignBoxes(int& heightOfBlock)
     
     computeLogicalBoxHeights(maxPositionTop, maxPositionBottom, maxAscent, maxDescent, strictMode);
 
-    if (maxAscent + maxDescent < QMAX(maxPositionTop, maxPositionBottom))
+    if (maxAscent + maxDescent < kMax(maxPositionTop, maxPositionBottom))
         adjustMaxAscentAndDescent(maxAscent, maxDescent, maxPositionTop, maxPositionBottom);
 
     int maxHeight = maxAscent + maxDescent;
@@ -323,7 +508,7 @@ void InlineFlowBox::adjustMaxAscentAndDescent(int& maxAscent, int& maxDescent,
                     maxAscent = curr->height() - maxDescent;
             }
 
-            if ( maxAscent + maxDescent >= QMAX( maxPositionTop, maxPositionBottom ) )
+            if (maxAscent + maxDescent >= kMax(maxPositionTop, maxPositionBottom))
                 break;
         }
 
@@ -479,8 +664,7 @@ void InlineFlowBox::shrinkBoxesWithNoTextChildren(int topPos, int bottomPos)
     }
 }
 
-void InlineFlowBox::paintBackgroundAndBorder(QPainter *p, int _x, int _y,
-                                             int _w, int _h, int _tx, int _ty, int xOffsetOnLine)
+void InlineFlowBox::paintBackgroundAndBorder(RenderObject::PaintInfo& i, int _tx, int _ty, int xOffsetOnLine)
 {
     // Move x/y to our coordinates.
     _tx += m_x;
@@ -489,13 +673,15 @@ void InlineFlowBox::paintBackgroundAndBorder(QPainter *p, int _x, int _y,
     int w = width();
     int h = height();
 
-    int my = QMAX(_ty,_y);
+    int my = kMax(_ty, i.r.y());
     int mh;
-    if (_ty<_y)
-        mh= QMAX(0,h-(_y-_ty));
+    if (_ty < i.r.y())
+        mh= kMax(0, h - (i.r.y() - _ty));
     else
-        mh = QMIN(_h,h);
+        mh = kMin(i.r.height(), h);
 
+    QPainter* p = i.p;
+    
     // You can use p::first-line to specify a background. If so, the root line boxes for
     // a line may actually have to paint a background.
     RenderStyle* styleToUse = object()->style(m_firstLine);
@@ -555,16 +741,38 @@ static bool shouldDrawDecoration(RenderObject* obj)
     return shouldDraw;
 }
 
-void InlineFlowBox::paintDecorations(QPainter *p, int _x, int _y,
-                                     int _w, int _h, int _tx, int _ty)
+void InlineFlowBox::paintDecorations(RenderObject::PaintInfo& i, int _tx, int _ty, bool paintedChildren)
 {
     // Now paint our text decorations. We only do this if we aren't in quirks mode (i.e., in
     // almost-strict mode or strict mode).
+    QPainter* p = i.p;
     _tx += m_x;
     _ty += m_y;
     RenderStyle* styleToUse = object()->style(m_firstLine);
     int deco = parent() ? styleToUse->textDecoration() : styleToUse->textDecorationsInEffect();
-    if (deco != TDNONE && shouldDrawDecoration(object())) {
+    if (deco != TDNONE && 
+        ((!paintedChildren && ((deco & UNDERLINE) || (deco & OVERLINE))) || (paintedChildren && (deco & LINE_THROUGH))) &&
+        shouldDrawDecoration(object())) {
+        int x = m_x + borderLeft() + paddingLeft();
+        int w = m_width - (borderLeft() + paddingLeft() + borderRight() + paddingRight());
+        RootInlineBox* rootLine = root();
+        if (rootLine->ellipsisBox()) {
+            int ellipsisX = rootLine->ellipsisBox()->xPos();
+            int ellipsisWidth = rootLine->ellipsisBox()->width();
+            
+            // FIXME: Will need to work with RTL
+            if (rootLine == this) {
+                if (x + w >= ellipsisX + ellipsisWidth)
+                    w -= (x + w - ellipsisX - ellipsisWidth);
+            }
+            else {
+                if (x >= ellipsisX)
+                    return;
+                if (x + w >= ellipsisX)
+                    w -= (x + w - ellipsisX);
+            }
+        }
+            
 #if APPLE_CHANGES
         // Set up the appropriate text-shadow effect for the decoration.
         // FIXME: Support multiple shadow effects.  Need more from the CG API before we can do this.
@@ -578,20 +786,24 @@ void InlineFlowBox::paintDecorations(QPainter *p, int _x, int _y,
         
         // We must have child boxes and have decorations defined.
         _tx += borderLeft() + paddingLeft();
-        int w = m_width - (borderLeft() + paddingLeft() + borderRight() + paddingRight());
+        
         QColor underline, overline, linethrough;
         underline = overline = linethrough = styleToUse->color();
         if (!parent())
             object()->getTextDecorationColors(deco, underline, overline, linethrough);
-        if (deco & UNDERLINE) {
+
+        if (styleToUse->font() != p->font())
+            p->setFont(styleToUse->font());
+
+        if (deco & UNDERLINE && !paintedChildren) {
             p->setPen(underline);
             p->drawLineForText(_tx, _ty, m_baseline, w);
         }
-        if (deco & OVERLINE) {
+        if (deco & OVERLINE && !paintedChildren) {
             p->setPen(overline);
             p->drawLineForText(_tx, _ty, 0, w);
         }
-        if (deco & LINE_THROUGH) {
+        if (deco & LINE_THROUGH && paintedChildren) {
             p->setPen(linethrough);
             p->drawLineForText(_tx, _ty, 2*m_baseline/3, w);
         }
@@ -602,3 +814,226 @@ void InlineFlowBox::paintDecorations(QPainter *p, int _x, int _y,
 #endif
     }
 }
+
+InlineBox* InlineFlowBox::firstLeafChild()
+{
+    InlineBox *box = firstChild();
+    while (box) {
+        InlineBox* next = 0;
+        if (!box->isInlineFlowBox())
+            break;
+        next = static_cast<InlineFlowBox*>(box)->firstChild();
+        if (!next)
+            break;
+        box = next;
+    }
+    return box;
+}
+
+InlineBox* InlineFlowBox::lastLeafChild()
+{
+    InlineBox *box = lastChild();
+    while (box) {
+        InlineBox* next = 0;
+        if (!box->isInlineFlowBox())
+            break;
+        next = static_cast<InlineFlowBox*>(box)->lastChild();
+        if (!next)
+            break;
+        box = next;
+    }
+    return box;
+}
+
+InlineBox* InlineFlowBox::closestChildForXPos(int _x, int _tx)
+{
+    if (_x < _tx + firstChild()->m_x)
+        // if the x coordinate is to the left of the first child
+        return firstChild(); 
+    else if (_x >= _tx + lastChild()->m_x + lastChild()->m_width)
+        // if the x coordinate is to the right of the last child
+        return lastChild(); 
+    else
+        // look for the closest child;
+        // check only the right edges, since the left edge of the first
+        // box has already been checked
+        for (InlineBox *box = firstChild(); box; box = box->nextOnLine())
+            if (_x < _tx + box->m_x + box->m_width)
+                return box;
+
+    return 0;
+}
+
+bool InlineFlowBox::canAccommodateEllipsis(bool ltr, int blockEdge, int ellipsisWidth)
+{
+    for (InlineBox *box = firstChild(); box; box = box->nextOnLine()) {
+        if (!box->canAccommodateEllipsis(ltr, blockEdge, ellipsisWidth))
+            return false;
+    }
+    return true;
+}
+
+int InlineFlowBox::placeEllipsisBox(bool ltr, int blockEdge, int ellipsisWidth, bool& foundBox)
+{
+    int result = -1;
+    for (InlineBox *box = firstChild(); box; box = box->nextOnLine()) {
+        int currResult = box->placeEllipsisBox(ltr, blockEdge, ellipsisWidth, foundBox);
+        if (currResult != -1 && result == -1)
+            result = currResult;
+    }
+    return result;
+}
+
+void InlineFlowBox::clearTruncation()
+{
+    for (InlineBox *box = firstChild(); box; box = box->nextOnLine())
+        box->clearTruncation();
+}
+
+void EllipsisBox::paint(RenderObject::PaintInfo& i, int _tx, int _ty)
+{
+    QPainter* p = i.p;
+    RenderStyle* _style = m_firstLine ? m_object->style(true) : m_object->style();
+    if (_style->font() != p->font())
+        p->setFont(_style->font());
+
+    const Font* font = &_style->htmlFont();
+    QColor textColor = _style->color();
+    if (textColor != p->pen().color())
+        p->setPen(textColor);
+    bool setShadow = false;
+    if (_style->textShadow()) {
+        p->setShadow(_style->textShadow()->x, _style->textShadow()->y,
+                     _style->textShadow()->blur, _style->textShadow()->color);
+        setShadow = true;
+    }
+    
+    const DOMString& str = m_str.string();
+    font->drawText(p, m_x + _tx, 
+                      m_y + _ty + m_baseline,
+                      (str.implementation())->s,
+                      str.length(), 0, str.length(),
+                      0, 
+                      QPainter::LTR, _style->visuallyOrdered());
+                      
+    if (setShadow)
+        p->clearShadow();
+    
+    if (m_markupBox) {
+        // Paint the markup box
+        _tx += m_x + m_width - m_markupBox->xPos();
+        _ty += m_y + m_baseline - (m_markupBox->yPos() + m_markupBox->baseline());
+        m_markupBox->object()->paint(i, _tx, _ty);
+    }
+}
+
+bool EllipsisBox::nodeAtPoint(RenderObject::NodeInfo& info, int _x, int _y, int _tx, int _ty,
+                              HitTestAction hitTestAction, bool inBox)
+{
+    if (m_markupBox) {
+        _tx += m_x + m_width - m_markupBox->xPos();
+        _ty += m_y + m_baseline - (m_markupBox->yPos() + m_markupBox->baseline());
+        inBox |= m_markupBox->object()->nodeAtPoint(info, _x, _y, _tx, _ty, hitTestAction, inBox);
+    }
+    
+    return inBox;
+}
+
+void RootInlineBox::detach(RenderArena* arena)
+{
+    detachEllipsisBox(arena);
+    InlineFlowBox::detach(arena);
+}
+
+void RootInlineBox::detachEllipsisBox(RenderArena* arena)
+{
+    if (m_ellipsisBox) {
+        m_ellipsisBox->detach(arena);
+        m_ellipsisBox = 0;
+    }
+}
+
+void RootInlineBox::clearTruncation()
+{
+    if (m_ellipsisBox) {
+        detachEllipsisBox(m_object->renderArena());
+        InlineFlowBox::clearTruncation();
+    }
+}
+
+bool RootInlineBox::canAccommodateEllipsis(bool ltr, int blockEdge, int lineBoxEdge, int ellipsisWidth)
+{
+    // First sanity-check the unoverflowed width of the whole line to see if there is sufficient room.
+    int delta = ltr ? lineBoxEdge - blockEdge : blockEdge - lineBoxEdge;
+    if (width() - delta < ellipsisWidth)
+        return false;
+
+    // Next iterate over all the line boxes on the line.  If we find a replaced element that intersects
+    // then we refuse to accommodate the ellipsis.  Otherwise we're ok.
+    return InlineFlowBox::canAccommodateEllipsis(ltr, blockEdge, ellipsisWidth);
+}
+
+void RootInlineBox::placeEllipsis(const AtomicString& ellipsisStr,  bool ltr, int blockEdge, int ellipsisWidth,
+                                  InlineBox* markupBox)
+{
+    // Create an ellipsis box.
+    m_ellipsisBox = new (m_object->renderArena()) EllipsisBox(m_object, ellipsisStr, this, 
+                                                              ellipsisWidth - (markupBox ? markupBox->width() : 0),
+                                                              yPos(), height(), baseline(), !prevRootBox(),
+                                                              markupBox);
+
+    if (ltr && (xPos() + width() + ellipsisWidth) <= blockEdge) {
+        m_ellipsisBox->m_x = xPos() + width();
+        return;
+    }
+
+    // Now attempt to find the nearest glyph horizontally and place just to the right (or left in RTL)
+    // of that glyph.  Mark all of the objects that intersect the ellipsis box as not painting (as being
+    // truncated).
+    bool foundBox = false;
+    m_ellipsisBox->m_x = placeEllipsisBox(ltr, blockEdge, ellipsisWidth, foundBox);
+}
+
+int RootInlineBox::placeEllipsisBox(bool ltr, int blockEdge, int ellipsisWidth, bool& foundBox)
+{
+    int result = InlineFlowBox::placeEllipsisBox(ltr, blockEdge, ellipsisWidth, foundBox);
+    if (result == -1)
+        result = ltr ? blockEdge - ellipsisWidth : blockEdge;
+    return result;
+}
+
+void RootInlineBox::paintEllipsisBox(RenderObject::PaintInfo& i, int _tx, int _ty) const
+{
+    if (m_ellipsisBox)
+        m_ellipsisBox->paint(i, _tx, _ty);
+}
+
+bool RootInlineBox::hitTestEllipsisBox(RenderObject::NodeInfo& info, int _x, int _y, int _tx, int _ty,
+                                       HitTestAction hitTestAction, bool inBox)
+{
+    if (m_ellipsisBox)
+        inBox |= m_ellipsisBox->nodeAtPoint(info, _x, _y, _tx, _ty, hitTestAction, inBox);
+    return inBox;
+}
+
+void RootInlineBox::adjustPosition(int dx, int dy)
+{
+    InlineFlowBox::adjustPosition(dx, dy);
+    m_topOverflow += dy;
+    m_bottomOverflow += dy;
+    m_blockHeight += dy;
+}
+
+void RootInlineBox::childRemoved(InlineBox* box)
+{
+    if (box->object() == m_lineBreakObj)
+        setLineBreakInfo(0,0);
+
+    RootInlineBox* prev = prevRootBox();
+    if (prev && prev->lineBreakObj() == box->object()) {
+        prev->setLineBreakInfo(0,0);
+        prev->markDirty();
+    }
+}
+
+}
