@@ -34,10 +34,7 @@
 #include "misc/helper.h"
 #include "rendering/render_style.h"
 #include "khtml_events.h"
-#include "xml/dom_nodeimpl.h"
-
-// Uncomment to turn on incremental repainting.
-#define INCREMENTAL_REPAINTING 1
+#include "xml/dom_docimpl.h"
 
 class QPainter;
 class QTextStream;
@@ -88,6 +85,7 @@ namespace DOM {
     class DocumentImpl;
     class ElementImpl;
     class EventImpl;
+    class Position;
 };
 
 namespace khtml {
@@ -116,6 +114,7 @@ public:
     virtual ~RenderObject();
 
     RenderObject *parent() const { return m_parent; }
+    bool hasAncestor(const RenderObject *obj) const;
 
     RenderObject *previousSibling() const { return m_previous; }
     RenderObject *nextSibling() const { return m_next; }
@@ -123,6 +122,15 @@ public:
     virtual RenderObject *firstChild() const { return 0; }
     virtual RenderObject *lastChild() const { return 0; }
 
+    RenderObject *nextRenderer() const; 
+    RenderObject *previousRenderer() const; 
+
+    RenderObject *nextEditable() const; 
+    RenderObject *previousEditable() const; 
+
+    RenderObject *firstLeafChild() const;
+    RenderObject *lastLeafChild() const;
+    
     virtual RenderLayer* layer() const { return 0; }
     RenderLayer* enclosingLayer();
     void addLayers(RenderLayer* parentLayer, RenderObject* newObject);
@@ -136,11 +144,9 @@ public:
     virtual QRect getOverflowClipRect(int tx, int ty) { return QRect(0,0,0,0); }
     virtual QRect getClipRect(int tx, int ty) { return QRect(0,0,0,0); }
     bool hasClip() { return isPositioned() &&  style()->hasClip(); }
-    bool hasOverflowClip() { return style()->hidesOverflow(); }
-
-    virtual int getBaselineOfFirstLineBox() { return -1; } // Tables and blocks implement this.
-    virtual InlineFlowBox* getFirstLineBox() { return 0; } // Tables and blocks implement this.
-
+    
+    virtual int getBaselineOfFirstLineBox() const { return -1; } 
+    
     // Obtains the nearest enclosing block (including this block) that contributes a first-line style to our inline
     // children.
     virtual RenderBlock* firstLineBlock() const;
@@ -159,7 +165,7 @@ public:
     // to determine its position.
     bool hasStaticX() const;
     bool hasStaticY() const;
-    virtual void setStaticX(short staticX) {};
+    virtual void setStaticX(int staticX) {};
     virtual void setStaticY(int staticY) {};
     
     // RenderObject tree manipulation
@@ -181,6 +187,9 @@ private:
     void setNextSibling(RenderObject *next) { m_next = next; }
     void setParent(RenderObject *parent) { m_parent = parent; }
     //////////////////////////////////////////
+    
+    QRect absoluteBoundingBoxRect();
+    void addAbsoluteRectForLayer(QRect& result);
 
 public:
     virtual const char *renderName() const { return "RenderObject"; }
@@ -236,6 +245,8 @@ public:
     virtual bool isTextArea() const { return false; }
     virtual bool isFrameSet() const { return false; }
     virtual bool isApplet() const { return false; }
+    
+    virtual bool isEditable() const;
 
     bool isHTMLMarquee() const;
     
@@ -254,6 +265,7 @@ public:
     bool isCompact() const { return style()->display() == COMPACT; } // compact object
     bool isRunIn() const { return style()->display() == RUN_IN; } // run-in object
     bool mouseInside() const;
+    bool isDragging() const;
     bool isReplaced() const { return m_replaced; } // a "replaced" element (see CSS)
     bool shouldPaintBackgroundOrBorder() const { return m_paintBackground; }
     bool needsLayout() const   { return m_needsLayout || m_normalChildNeedsLayout || m_posChildNeedsLayout; }
@@ -261,12 +273,21 @@ public:
     bool posChildNeedsLayout() const { return m_posChildNeedsLayout; }
     bool normalChildNeedsLayout() const { return m_normalChildNeedsLayout; }
     bool minMaxKnown() const{ return m_minMaxKnown; }
-    bool overhangingContents() const { return m_overhangingContents; }
     bool isSelectionBorder() const { return m_isSelectionBorder; }
     bool recalcMinMax() const { return m_recalcMinMax; }
 
+    bool hasOverflowClip() const { return m_hasOverflowClip; }
+    bool hasAutoScrollbars() const { return hasOverflowClip() && 
+        (style()->overflow() == OAUTO || style()->overflow() == OOVERLAY); }
+    bool scrollsOverflow() const { return hasOverflowClip() &&
+        (style()->overflow() == OSCROLL || hasAutoScrollbars()); }
+    bool includeScrollbarSize() const { return hasOverflowClip() &&
+        (style()->overflow() == OSCROLL || style()->overflow() == OAUTO); }
+
     RenderStyle* getPseudoStyle(RenderStyle::PseudoId pseudo, RenderStyle* parentStyle = 0) const;
     
+    void updateDragState(bool dragOn);
+
     RenderCanvas* canvas() const;
 
     // don't even think about making this method virtual!
@@ -281,8 +302,6 @@ public:
      */
     RenderObject *container() const;
 
-    void setOverhangingContents(bool p=true);
-
     virtual void markAllDescendantsWithFloatsForLayout(RenderObject* floatToRemove = 0);
     void markContainingBlocksForLayout();
     void setNeedsLayout(bool b, bool markParents = true);
@@ -314,14 +333,21 @@ public:
     void setRenderText() { m_isText = true; }
     void setReplaced(bool b=true) { m_replaced = b; }
     void setIsSelectionBorder(bool b=true) { m_isSelectionBorder = b; }
+    void setHasOverflowClip(bool b = true) { m_hasOverflowClip = b; }
 
-#ifdef INCREMENTAL_REPAINTING
     void scheduleRelayout();
-#else
-    void scheduleRelayout(RenderObject* clippedObj);
-#endif
     
-    virtual InlineBox* createInlineBox(bool makePlaceHolderBox, bool isRootLineBox);
+    virtual InlineBox* createInlineBox(bool makePlaceHolderBox, bool isRootLineBox, bool isOnlyRun=false);
+    virtual void dirtyLineBoxes(bool fullLayout, bool isRootLineBox=false);
+    
+    // For inline replaced elements, this function returns the inline box that owns us.  Enables
+    // the replaced RenderObject to quickly determine what line it is contained on and to easily
+    // iterate over structures on the line.
+    virtual InlineBox* inlineBoxWrapper() const;
+    virtual void setInlineBoxWrapper(InlineBox* b);
+    void deleteLineBoxWrapper();
+
+    virtual InlineBox *inlineBox(long offset=0);
     
     // for discussion of lineHeight see CSS2 spec
     virtual short lineHeight( bool firstLine, bool isRootLineBox=false ) const;
@@ -335,18 +361,20 @@ public:
      * Paint the object and its children, clipped by (x|y|w|h).
      * (tx|ty) is the calculated position of the parent
      */
-    virtual void paint(QPainter *p, int x, int y, int w, int h, int tx, int ty, 
-                       PaintAction paintAction);
-
-    virtual void paintObject( QPainter */*p*/, int /*x*/, int /*y*/,
-                              int /*w*/, int /*h*/, int /*tx*/, int /*ty*/,
-                              PaintAction paintAction /*paintAction*/) {}
+    struct PaintInfo {
+        PaintInfo(QPainter* _p, const QRect& _r, PaintAction _phase, RenderObject *_paintingRoot)
+        : p(_p), r(_r), phase(_phase), paintingRoot(_paintingRoot) {}
+        QPainter* p;
+        QRect     r;
+        PaintAction phase;
+        RenderObject *paintingRoot;      // used to draw just one element and its visual kids
+    };
+    virtual void paint(PaintInfo& i, int tx, int ty);
     void paintBorder(QPainter *p, int _tx, int _ty, int w, int h, const RenderStyle* style, bool begin=true, bool end=true);
     void paintOutline(QPainter *p, int _tx, int _ty, int w, int h, const RenderStyle* style);
 
     // RenderBox implements this.
-    virtual void paintBoxDecorations(QPainter *p,int _x, int _y,
-                                     int _w, int _h, int _tx, int _ty) {};
+    virtual void paintBoxDecorations(PaintInfo& i, int _tx, int _ty) {};
 
     virtual void paintBackgroundExtended(QPainter *p, const QColor &c, CachedImage *bg, int clipy, int cliph,
                                          int _tx, int _ty, int w, int height,
@@ -446,11 +474,22 @@ public:
         bool m_active;
     };
 
-    FindSelectionResult checkSelectionPoint(int x, int y, int tx, int ty, DOM::NodeImpl*&, int& offset);
-    virtual FindSelectionResult checkSelectionPointIgnoringContinuations(int x, int y, int tx, int ty, DOM::NodeImpl*&, int& offset);
+    // Used to signal a specific subrect within an object that must be repainted after
+    // layout is complete.
+    struct RepaintInfo {
+        RenderObject* m_object;
+        QRect m_repaintRect;
+    
+        RepaintInfo(RenderObject* o, const QRect& r) :m_object(o), m_repaintRect(r) {}
+    };
+    
     virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty,
                              HitTestAction hitTestAction = HitTestAll, bool inside=false);
     
+    virtual DOM::Position positionForCoordinates(int x, int y);
+    
+    virtual void dirtyLinesFromChangedChild(RenderObject* child);
+    
     // set the style of the object.
     virtual void setStyle(RenderStyle *style);
 
@@ -458,18 +497,24 @@ public:
     RenderBlock *containingBlock() const;
 
     // return just the width of the containing block
-    virtual short containingBlockWidth() const;
+    virtual int containingBlockWidth() const;
     // return just the height of the containing block
     virtual int containingBlockHeight() const;
 
     // size of the content area (box size minus padding/border)
-    virtual short contentWidth() const { return 0; }
+    virtual int contentWidth() const { return 0; }
     virtual int contentHeight() const { return 0; }
 
     // intrinsic extend of replaced elements. undefined otherwise
-    virtual short intrinsicWidth() const { return 0; }
+    virtual int intrinsicWidth() const { return 0; }
     virtual int intrinsicHeight() const { return 0; }
 
+    // used by flexible boxes to impose a flexed width/height override
+    virtual int overrideSize() const { return 0; }
+    virtual int overrideWidth() const { return 0; }
+    virtual int overrideHeight() const { return 0; }
+    virtual void setOverrideSize(int s) {}
+
     // relative to parent node
     virtual void setPos( int /*xPos*/, int /*yPos*/ ) { }
     virtual void setWidth( int /*width*/ ) { }
@@ -482,7 +527,7 @@ public:
     virtual bool absolutePosition(int &/*xPos*/, int &/*yPos*/, bool fixed = false);
 
     // width and height are without margins but include paddings and borders
-    virtual short width() const { return 0; }
+    virtual int width() const { return 0; }
     virtual int height() const { return 0; }
 
     // The height of a block when you include normal flow overflow spillage out of the bottom
@@ -495,7 +540,7 @@ public:
 
     // IE extensions. Used to calculate offsetWidth/Height.  Overridden by inlines (render_flow) 
     // to return the remaining width on a given line (and the height of a single line). -dwh
-    virtual short offsetWidth() const { return width(); }
+    virtual int offsetWidth() const { return width(); }
     virtual int offsetHeight() const { return height(); }
     
     // IE exxtensions.  Also supported by Gecko.  We override in render flow to get the
@@ -506,12 +551,12 @@ public:
 
     // More IE extensions.  clientWidth and clientHeight represent the interior of an object
     // excluding border and scrollbar.
-    short clientWidth() const;
+    int clientWidth() const;
     int clientHeight() const;
 
     // scrollWidth/scrollHeight will be the same as clientWidth/clientHeight unless the
     // object has overflow:hidden/scroll/auto specified and also has overflow.
-    short scrollWidth() const;
+    int scrollWidth() const;
     int scrollHeight() const;
 
     // The following seven functions are used to implement collapsing margins.
@@ -570,12 +615,15 @@ public:
 
     virtual void absoluteRects(QValueList<QRect>& rects, int _tx, int _ty);
 
+    // the rect that will be painted if this object is passed as the paintingRoot
+    QRect paintingRootRect(QRect& topLevelRect);
+
 #if APPLE_CHANGES
     virtual void addFocusRingRects(QPainter *painter, int _tx, int _ty);
 #endif
 
-    virtual short minWidth() const { return 0; }
-    virtual short maxWidth() const { return 0; }
+    virtual int minWidth() const { return 0; }
+    virtual int maxWidth() const { return 0; }
 
     RenderStyle* style() const { return m_style; }
     RenderStyle* style( bool firstLine ) const;
@@ -602,9 +650,8 @@ public:
     // Repaint a specific subrectangle within a given object.  The rect |r| is in the object's coordinate space.
     void repaintRectangle(const QRect& r, bool immediate = false);
     
-#ifdef INCREMENTAL_REPAINTING
     // Repaint only if our old bounds and new bounds are different.
-    virtual void repaintAfterLayoutIfNeeded(const QRect& oldBounds, const QRect& oldFullBounds);
+    bool repaintAfterLayoutIfNeeded(const QRect& oldBounds, const QRect& oldFullBounds);
 
     // Repaint only if the object moved.
     virtual void repaintDuringLayoutIfMoved(int oldX, int oldY);
@@ -616,7 +663,6 @@ public:
     virtual void repaintObjectsBeforeLayout();
 
     bool checkForRepaintDuringLayout() const;
-#endif
 
     // Returns the rect that should be repainted whenever this object changes.  The rect is in the view's
     // coordinate space.  This method deals with outlines and overflow.
@@ -624,9 +670,7 @@ public:
 
     QRect getAbsoluteRepaintRectWithOutline(int ow);
 
-#ifdef INCREMENTAL_REPAINTING
     virtual void getAbsoluteRepaintRectIncludingFloats(QRect& bounds, QRect& boundsWithChildren);
-#endif
 
     // Given a rect in the object's coordinate space, this method converts the rectangle to the view's
     // coordinate space.
@@ -658,8 +702,21 @@ public:
 
     virtual SelectionState selectionState() const { return SelectionNone;}
     virtual void setSelectionState(SelectionState) {}
+    bool shouldSelect() const;
 
-    virtual void cursorPos(int /*offset*/, int &/*_x*/, int &/*_y*/, int &/*height*/);
+    DOM::NodeImpl* draggableNode(bool dhtmlOK, bool uaOK, bool& dhtmlWillDrag) const;
+
+    /**
+     * Returns the content coordinates of the caret within this render object.
+     * @param offset zero-based offset determining position within the render object.
+     * @param override @p true if input overrides existing characters,
+     * @p false if it inserts them. The width of the caret depends on this one.
+     * @param _x returns the left coordinate
+     * @param _y returns the top coordinate
+     * @param width returns the caret's width
+     * @param height returns the caret's height
+     */
+    virtual void caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height);
 
     virtual int lowestPosition(bool includeOverflowInterior=true, bool includeSelf=true) const { return 0; }
     virtual int rightmostPosition(bool includeOverflowInterior=true, bool includeSelf=true) const { return 0; }
@@ -668,6 +725,10 @@ public:
     virtual void calcVerticalMargins() {}
     void removeFromObjectLists();
 
+    // When performing a global document tear-down, the renderer of the document is cleared.  We use this
+    // as a hook to detect the case of document destruction and don't waste time doing unnecessary work.
+    bool documentBeingDestroyed() const { return !document()->renderer(); }
+
     virtual void detach();
 
     const QFont &font(bool firstLine) const {
@@ -686,6 +747,10 @@ public:
     // Convenience, to avoid repeating the code to dig down to get this.
     QChar backslashAsCurrencySymbol() const;
 
+    virtual long caretMinOffset() const;
+    virtual long caretMaxOffset() const;
+    virtual unsigned long caretMaxRenderedOffset() const;
+
     virtual void setPixmap(const QPixmap&, const QRect&, CachedImage *);
 
 protected:
@@ -705,6 +770,14 @@ protected:
     
     void arenaDelete(RenderArena *arena);
 
+    RenderObject *paintingRootForChildren(PaintInfo &i) const {
+        // if we're the painting root, kids draw normally, and see root of 0
+        return (!i.paintingRoot || i.paintingRoot == this) ? 0 : i.paintingRoot;
+    }
+    bool shouldPaintWithinRoot(PaintInfo &i) const {
+        return !i.paintingRoot || i.paintingRoot == this;
+    }
+
 private:
     RenderStyle* m_style;
 
@@ -714,7 +787,7 @@ private:
     RenderObject *m_previous;
     RenderObject *m_next;
 
-    short m_verticalPosition;
+    mutable short m_verticalPosition;
 
     bool m_needsLayout               : 1;
     bool m_normalChildNeedsLayout    : 1;
@@ -723,7 +796,6 @@ private:
     bool m_floating                  : 1;
 
     bool m_positioned                : 1;
-    bool m_overhangingContents       : 1;
     bool m_relPositioned             : 1;
     bool m_paintBackground           : 1; // if the box has something to paint in the
                                           // background painting phase (background, border, etc)
@@ -734,8 +806,11 @@ private:
     bool m_inline                    : 1;
     bool m_replaced                  : 1;
     bool m_mouseInside               : 1;
+    bool m_isDragging                : 1;
     bool m_isSelectionBorder         : 1;
 
+    bool m_hasOverflowClip           : 1;
+
     void arenaDelete(RenderArena *arena, void *objectBase);
 
     // note: do not add unnecessary bitflags, we have 32 bit already!
