@@ -22,9 +22,10 @@
  *
  */
 #include "render_replaced.h"
-#include "render_canvas.h"
 
 #include "render_arena.h"
+#include "render_canvas.h"
+#include "render_line.h"
 
 #include <assert.h>
 #include <qwidget.h>
@@ -37,6 +38,7 @@
 #include "xml/dom2_eventsimpl.h"
 #include "khtml_part.h"
 #include "xml/dom_docimpl.h" // ### remove dependency
+#include "xml/dom_position.h"
 #include <kdebug.h>
 
 using namespace khtml;
@@ -53,25 +55,28 @@ RenderReplaced::RenderReplaced(DOM::NodeImpl* node)
     m_intrinsicHeight = 150;
 }
 
-void RenderReplaced::paint(QPainter *p, int _x, int _y, int _w, int _h,
-                           int _tx, int _ty, PaintAction paintAction)
+bool RenderReplaced::shouldPaint(PaintInfo& i, int& _tx, int& _ty)
 {
-    if (paintAction != PaintActionForeground)
-        return;
+    if (i.phase != PaintActionForeground && i.phase != PaintActionOutline && i.phase != PaintActionSelection)
+        return false;
+
+    if (!shouldPaintWithinRoot(i))
+        return false;
         
     // if we're invisible or haven't received a layout yet, then just bail.
-    if (style()->visibility() != VISIBLE || m_y <=  -500000)  return;
+    if (style()->visibility() != VISIBLE || m_y <=  -500000)  return false;
 
-    _tx += m_x;
-    _ty += m_y;
+    int tx = _tx + m_x;
+    int ty = _ty + m_y;
 
-    if((_ty > _y + _h) || (_ty + m_height < _y))
-        return;
+    // Early exit if the element touches the edges.
+    int os = 2*maximalOutlineSize(i.phase);
+    if ((tx >= i.r.x() + i.r.width() + os) || (tx + m_width <= i.r.x() - os))
+        return false;
+    if ((ty >= i.r.y() + i.r.height() + os) || (ty + m_height <= i.r.y() - os))
+        return false;
 
-    if(shouldPaintBackgroundOrBorder()) 
-        paintBoxDecorations(p, _x, _y, _w, _h, _tx, _ty);
-
-    paintObject(p, _x, _y, _w, _h, _tx, _ty, paintAction);
+    return true;
 }
 
 void RenderReplaced::calcMinMaxWidth()
@@ -82,11 +87,7 @@ void RenderReplaced::calcMinMaxWidth()
     kdDebug( 6040 ) << "RenderReplaced::calcMinMaxWidth() known=" << minMaxKnown() << endl;
 #endif
 
-    int width = calcReplacedWidth();
-
-    if (!isWidget())
-        width += paddingLeft() + paddingRight() + borderLeft() + borderRight();
-
+    int width = calcReplacedWidth() + paddingLeft() + paddingRight() + borderLeft() + borderRight();
     if ( style()->width().isPercent() || style()->height().isPercent() ) {
         m_minWidth = 0;
         m_maxWidth = width;
@@ -97,12 +98,12 @@ void RenderReplaced::calcMinMaxWidth()
     setMinMaxKnown();
 }
 
-short RenderReplaced::lineHeight( bool ) const
+short RenderReplaced::lineHeight( bool, bool ) const
 {
     return height()+marginTop()+marginBottom();
 }
 
-short RenderReplaced::baselinePosition( bool ) const
+short RenderReplaced::baselinePosition( bool, bool ) const
 {
     return height()+marginTop()+marginBottom();
 }
@@ -113,6 +114,52 @@ bool RenderReplaced::canHaveChildren() const
     return false;
 }
 
+long RenderReplaced::caretMinOffset() const 
+{ 
+    return 0; 
+}
+
+// Returns 1 since a replaced element can have the caret positioned 
+// at its beginning (0), or at its end (1).
+long RenderReplaced::caretMaxOffset() const 
+{ 
+    return 1; 
+}
+
+unsigned long RenderReplaced::caretMaxRenderedOffset() const
+{
+    return 1; 
+}
+
+Position RenderReplaced::positionForCoordinates(int _x, int _y)
+{
+    InlineBox *box = inlineBoxWrapper();
+    if (!box)
+        return Position(element(), 0);
+
+    RootInlineBox *root = box->root();
+
+    int absx, absy;
+    containingBlock()->absolutePosition(absx, absy);
+
+    int top = absy + root->topOverflow();
+    int bottom = root->nextRootBox() ? absy + root->nextRootBox()->topOverflow() : absy + root->bottomOverflow();
+
+    if (_y < top)
+        return Position(element(), caretMinOffset()); // coordinates are above
+    
+    if (_y >= bottom)
+        return Position(element(), caretMaxOffset()); // coordinates are below
+    
+    if (element()) {
+        if (_x <= absx + xPos() + (width() / 2))
+            return Position(element(), 0);
+        return Position(element(), 1);
+    }
+
+    return RenderBox::positionForCoordinates(_x, _y);
+}
+
 // -----------------------------------------------------------------------------
 
 RenderWidget::RenderWidget(DOM::NodeImpl* node)
@@ -130,7 +177,7 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
     ref();
 }
 
-void RenderWidget::detach(RenderArena* renderArena)
+void RenderWidget::detach()
 {
     remove();
 
@@ -141,10 +188,15 @@ void RenderWidget::detach(RenderArena* renderArena)
         m_widget->removeEventFilter( this );
         m_widget->setMouseTracking( false );
     }
-    
-    m_node = 0;
-    
-    deref(renderArena);
+
+    RenderArena* arena = renderArena();
+    if (m_inlineBoxWrapper) {
+        if (!documentBeingDestroyed())
+            m_inlineBoxWrapper->remove();
+        m_inlineBoxWrapper->detach(arena);
+    }
+    setNode(0);
+    deref(arena);
 }
 
 RenderWidget::~RenderWidget()
@@ -190,14 +242,24 @@ void RenderWidget::setQWidget(QWidget *widget, bool deleteWidget)
             connect( m_widget, SIGNAL( destroyed()), this, SLOT( slotWidgetDestructed()));
             m_widget->installEventFilter(this);
             // if we've already received a layout, apply the calculated space to the
-            // widget immediately
-            if (!needsLayout()) {
+            // widget immediately, but we have to have really been full constructed (with a non-null
+            // style pointer).
+            if (!needsLayout() && style()) {
 		resizeWidget( m_widget,
 			      m_width-borderLeft()-borderRight()-paddingLeft()-paddingRight(),
 			      m_height-borderLeft()-borderRight()-paddingLeft()-paddingRight() );
             }
             else
                 setPos(xPos(), -500000);
+
+#if APPLE_CHANGES
+	    if (style()) {
+	        if (style()->visibility() != VISIBLE)
+                    m_widget->hide();
+		else
+		    m_widget->show();
+	    }
+#endif
         }
 	m_view->addChild( m_widget, -500000, 0 );
     }
@@ -208,11 +270,13 @@ void RenderWidget::layout( )
 {
     KHTMLAssert( needsLayout() );
     KHTMLAssert( minMaxKnown() );
+#if !APPLE_CHANGES
     if ( m_widget ) {
 	resizeWidget( m_widget,
 		      m_width-borderLeft()-borderRight()-paddingLeft()-paddingRight(),
 		      m_height-borderLeft()-borderRight()-paddingLeft()-paddingRight() );
     }
+#endif
 
     setNeedsLayout(false);
 }
@@ -236,29 +300,49 @@ void RenderWidget::slotWidgetDestructed()
 void RenderWidget::setStyle(RenderStyle *_style)
 {
     RenderReplaced::setStyle(_style);
-    if(m_widget)
+    if (m_widget)
     {
         m_widget->setFont(style()->font());
-        if (style()->visibility() != VISIBLE) {
+        if (style()->visibility() != VISIBLE)
             m_widget->hide();
-        }
+#if APPLE_CHANGES
+        else
+            m_widget->show();
+#endif
     }
-
-    // do not paint background or borders for widgets
-    setShouldPaintBackgroundOrBorder(false);
 }
 
-#if APPLE_CHANGES
-void RenderWidget::paintObject(QPainter *p, int x, int y, int width, int height, int _tx, int _ty,
-                               PaintAction paintAction)
-#else
-void RenderWidget::paintObject(QPainter* /*p*/, int, int, int, int, int _tx, int _ty,
-                               PaintAction paintAction)
-#endif
+void RenderWidget::paint(PaintInfo& i, int _tx, int _ty)
 {
-    if (!m_widget || !m_view || paintAction != PaintActionForeground)
+    if (!shouldPaint(i, _tx, _ty)) return;
+
+    _tx += m_x;
+    _ty += m_y;
+    
+    if (shouldPaintBackgroundOrBorder() && i.phase != PaintActionOutline) 
+        paintBoxDecorations(i, _tx, _ty);
+
+#if APPLE_CHANGES
+    if (!m_widget || !m_view || i.phase != PaintActionForeground ||
+        style()->visibility() != VISIBLE)
         return;
 
+    // Move the widget if necessary.  We normally move and resize widgets during layout, but sometimes
+    // widgets can move without layout occurring (most notably when you scroll a document that
+    // contains fixed positioned elements).
+    int newX = _tx + borderLeft() + paddingLeft();
+    int newY = _ty + borderTop() + paddingTop();
+    if (m_view->childX(m_widget) != newX || m_view->childY(m_widget) != newY)
+        m_widget->move(newX, newY);
+    
+    // Tell the widget to paint now.  This is the only time the widget is allowed
+    // to paint itself.  That way it will composite properly with z-indexed layers.
+    m_widget->paint(i.p, i.r);
+    
+#else
+    if (!m_widget || !m_view || i.phase != PaintActionForeground)
+        return;
+    
     if (style()->visibility() != VISIBLE) {
         m_widget->hide();
         return;
@@ -267,7 +351,6 @@ void RenderWidget::paintObject(QPainter* /*p*/, int, int, int, int, int _tx, int
     int xPos = _tx+borderLeft()+paddingLeft();
     int yPos = _ty+borderTop()+paddingTop();
 
-#if !APPLE_CHANGES
     int childw = m_widget->width();
     int childh = m_widget->height();
     if ( (childw == 2000 || childh == 3072) && m_widget->inherits( "KHTMLView" ) ) {
@@ -291,8 +374,8 @@ void RenderWidget::paintObject(QPainter* /*p*/, int, int, int, int, int _tx, int
 	    }
 // 	    qDebug("calculated yNew=%d", yNew);
 	}
-	yNew = QMIN( yNew, yPos + m_height - childh );
-	yNew = QMAX( yNew, yPos );
+	yNew = kMin( yNew, yPos + m_height - childh );
+	yNew = kMax( yNew, yPos );
 	if ( yNew != childy || xNew != childx ) {
 	    if ( vw->contentsHeight() < yNew - yPos + childh )
 		vw->resizeContents( vw->contentsWidth(), yNew - yPos + childh );
@@ -301,15 +384,9 @@ void RenderWidget::paintObject(QPainter* /*p*/, int, int, int, int, int _tx, int
 	xPos = xNew;
 	yPos = yNew;
     }
-#endif
 
     m_view->addChild(m_widget, xPos, yPos );
     m_widget->show();
-    
-#if APPLE_CHANGES
-    // Tell the widget to paint now.  This is the only time the widget is allowed
-    // to paint itself.  That way it will composite properly with z-indexed layers.
-    m_widget->paint(p, QRect(x, y, width, height));
 #endif
 }
 
@@ -322,7 +399,8 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
     if ( !element() ) return true;
 
     RenderArena *arena = ref();
-    element()->ref();
+    DOM::NodeImpl *elem = element();
+    elem->ref();
 
     bool filtered = false;
 
@@ -336,9 +414,10 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
         if ( QFocusEvent::reason() != QFocusEvent::Popup )
        {
            //kdDebug(6000) << "RenderWidget::eventFilter captures FocusOut" << endl;
-            element()->dispatchHTMLEvent(EventImpl::BLUR_EVENT,false,false);
-//             if (  element()->isEditable() ) {
-//                 KHTMLPartBrowserExtension *ext = static_cast<KHTMLPartBrowserExtension *>( element()->view->part()->browserExtension() );
+            if (elem->getDocument()->focusNode() == elem)
+                elem->getDocument()->setFocusNode(0);
+//             if (  elem->isEditable() ) {
+//                 KHTMLPartBrowserExtension *ext = static_cast<KHTMLPartBrowserExtension *>( elem->view->part()->browserExtension() );
 //                 if ( ext )  ext->editableWidgetBlurred( m_widget );
 //             }
 	    handleFocusOut();
@@ -346,9 +425,9 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
         break;
     case QEvent::FocusIn:
         //kdDebug(6000) << "RenderWidget::eventFilter captures FocusIn" << endl;
-        element()->getDocument()->setFocusNode(element());
+        elem->getDocument()->setFocusNode(elem);
 //         if ( isEditable() ) {
-//             KHTMLPartBrowserExtension *ext = static_cast<KHTMLPartBrowserExtension *>( element()->view->part()->browserExtension() );
+//             KHTMLPartBrowserExtension *ext = static_cast<KHTMLPartBrowserExtension *>( elem->view->part()->browserExtension() );
 //             if ( ext )  ext->editableWidgetFocused( m_widget );
 //         }
         break;
@@ -364,20 +443,20 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
 //         m_state  = _e->state();
 //         QMouseEvent e2(e->type(),QPoint(absX,absY)+_e->pos(),_e->button(),_e->state());
 
-//         element()->dispatchMouseEvent(&e2,EventImpl::MOUSEUP_EVENT,m_clickCount);
+//         elem->dispatchMouseEvent(&e2,EventImpl::MOUSEUP_EVENT,m_clickCount);
 
 //         if((m_mousePos - e2.pos()).manhattanLength() <= QApplication::startDragDistance()) {
 //             // DOM2 Events section 1.6.2 says that a click is if the mouse was pressed
 //             // and released in the "same screen location"
 //             // As people usually can't click on the same pixel, we're a bit tolerant here
-//             element()->dispatchMouseEvent(&e2,EventImpl::CLICK_EVENT,m_clickCount);
+//             elem->dispatchMouseEvent(&e2,EventImpl::CLICK_EVENT,m_clickCount);
 //         }
 
 //         if(!isRenderButton()) {
 //             // ### DOMActivate is also dispatched for thigs like selects & textareas -
 //             // not sure if this is correct
-//             element()->dispatchUIEvent(EventImpl::DOMACTIVATE_EVENT,m_isDoubleClick ? 2 : 1);
-//             element()->dispatchMouseEvent(&e2, m_isDoubleClick ? EventImpl::KHTML_DBLCLICK_EVENT : EventImpl::KHTML_CLICK_EVENT, m_clickCount);
+//             elem->dispatchUIEvent(EventImpl::DOMACTIVATE_EVENT,m_isDoubleClick ? 2 : 1);
+//             elem->dispatchMouseEvent(&e2, m_isDoubleClick ? EventImpl::KHTML_DBLCLICK_EVENT : EventImpl::KHTML_CLICK_EVENT, m_clickCount);
 //             m_isDoubleClick = false;
 //         }
 //         else
@@ -397,21 +476,21 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
 //         absolutePosition(absX,absY);
 //         QMouseEvent* _e = static_cast<QMouseEvent*>(e);
 //         QMouseEvent e2(e->type(),QPoint(absX,absY)+_e->pos(),_e->button(),_e->state());
-//         element()->dispatchMouseEvent(&e2);
+//         elem->dispatchMouseEvent(&e2);
 //         // ### change cursor like in KHTMLView?
 //     }
     break;
     case QEvent::KeyPress:
     case QEvent::KeyRelease:
     {
-        if (!element()->dispatchKeyEvent(static_cast<QKeyEvent*>(e)))
+        if (!elem->dispatchKeyEvent(static_cast<QKeyEvent*>(e)))
             filtered = true;
         break;
     }
     default: break;
     };
 
-    element()->deref();
+    elem->deref();
 
     // stop processing if the widget gets deleted, but continue in all other cases
     if (hasOneRef())
@@ -428,4 +507,45 @@ void RenderWidget::deref(RenderArena *arena)
         arenaDelete(arena);
 }
 
+#if APPLE_CHANGES
+void RenderWidget::updateWidgetPositions()
+{
+    if (!m_widget)
+        return;
+    
+    int x, y, width, height;
+    absolutePosition(x,y);
+    x += borderLeft() + paddingLeft();
+    y += borderTop() + paddingTop();
+    width = m_width - borderLeft() - borderRight() - paddingLeft() - paddingRight();
+    height = m_height - borderTop() - borderBottom() - paddingTop() - paddingBottom();
+    QRect newBounds(x,y,width,height);
+    QRect oldBounds(m_widget->frameGeometry());
+    if (newBounds != oldBounds) {
+        // The widget changed positions.  Update the frame geometry.
+        if (checkForRepaintDuringLayout()) {
+            RenderCanvas* c = canvas();
+            if (!c->printingMode()) {
+                c->repaintViewRectangle(oldBounds);
+                c->repaintViewRectangle(newBounds);
+            }
+        }
+
+        RenderArena *arena = ref();
+        element()->ref();
+        m_widget->setFrameGeometry(newBounds);
+        element()->deref();
+        deref(arena);
+    }
+}
+#endif
+
+void RenderWidget::setSelectionState(SelectionState s) 
+{
+    if (m_selectionState != s) {
+        m_selectionState = s;
+        m_widget->setIsSelected(m_selectionState != SelectionNone);
+    }
+}
+
 #include "render_replaced.moc"
