@@ -34,9 +34,9 @@ static const CharsetEntry table[] = {
     { "unicode", kCFStringEncodingUnicode, NoEncodingFlags },
     { "utf16", kCFStringEncodingUnicode, NoEncodingFlags },
     { "UTF-16BE", kCFStringEncodingUnicode, BigEndian },
-    { "unicodefeff", kCFStringEncodingUnicode, BigEndian },
+    { "unicodefffe", kCFStringEncodingUnicode, BigEndian },
     { "UTF-16LE", kCFStringEncodingUnicode, LittleEndian },
-    { "unicodefffe", kCFStringEncodingUnicode, LittleEndian },
+    { "unicodefeff", kCFStringEncodingUnicode, LittleEndian },
     { "UTF-8", kCFStringEncodingUTF8, NoEncodingFlags },
     { "unicode11utf8", kCFStringEncodingUTF8, NoEncodingFlags },
     { "unicode20utf8", kCFStringEncodingUTF8, NoEncodingFlags },
@@ -49,6 +49,7 @@ static const CharsetEntry table[] = {
     { "x-mac-hebrew", kCFStringEncodingMacHebrew, NoEncodingFlags },
     { "x-mac-greek", kCFStringEncodingMacGreek, NoEncodingFlags },
     { "x-mac-cyrillic", kCFStringEncodingMacCyrillic, NoEncodingFlags },
+    { "xmacukrainian", kCFStringEncodingMacCyrillic, NoEncodingFlags },
     { "x-mac-devanagari", kCFStringEncodingMacDevanagari, NoEncodingFlags },
     { "x-mac-gurmukhi", kCFStringEncodingMacGurmukhi, NoEncodingFlags },
     { "x-mac-gujarati", kCFStringEncodingMacGujarati, NoEncodingFlags },
@@ -65,7 +66,6 @@ static const CharsetEntry table[] = {
     { "x-mac-icelandic", kCFStringEncodingMacIcelandic, NoEncodingFlags },
     { "x-mac-romanian", kCFStringEncodingMacRomanian, NoEncodingFlags },
     { "x-mac-farsi", kCFStringEncodingMacFarsi, NoEncodingFlags },
-    { "x-mac-ukrainian", kCFStringEncodingMacUkrainian, NoEncodingFlags },
     { "x-mac-vt100", kCFStringEncodingMacVT100, NoEncodingFlags },
     { "ISO-8859-2", kCFStringEncodingISOLatin2, NoEncodingFlags },
     { "csisolatin2", kCFStringEncodingISOLatin2, NoEncodingFlags },
@@ -108,12 +108,10 @@ static const CharsetEntry table[] = {
     { "ISO-8859-8-I", kCFStringEncodingISOLatinHebrew, NoEncodingFlags },
     { "csiso88598i", kCFStringEncodingISOLatinHebrew, NoEncodingFlags },
     { "logical", kCFStringEncodingISOLatinHebrew, NoEncodingFlags },
-    { "ISO-8859-8-E", kCFStringEncodingISOLatinHebrew, VisualOrdering },
-    { "csiso88598e", kCFStringEncodingISOLatinHebrew, VisualOrdering },
+    { "ISO-8859-8", kCFStringEncodingISOLatinHebrew, VisualOrdering },
     { "csisolatinhebrew", kCFStringEncodingISOLatinHebrew, VisualOrdering },
     { "dos862", kCFStringEncodingISOLatinHebrew, VisualOrdering },
     { "hebrew", kCFStringEncodingISOLatinHebrew, VisualOrdering },
-    { "iso88598", kCFStringEncodingISOLatinHebrew, VisualOrdering },
     { "iso885981988", kCFStringEncodingISOLatinHebrew, VisualOrdering },
     { "isoir138", kCFStringEncodingISOLatinHebrew, VisualOrdering },
     { "visual", kCFStringEncodingISOLatinHebrew, VisualOrdering },
@@ -141,6 +139,11 @@ static const CharsetEntry table[] = {
     { "csisolatin9", kCFStringEncodingISOLatin9, NoEncodingFlags },
     { "l9", kCFStringEncodingISOLatin9, NoEncodingFlags },
     { "latin9", kCFStringEncodingISOLatin9, NoEncodingFlags },
+    { "ISO-8859-16", kCFStringEncodingISOLatin10, NoEncodingFlags },
+    { "iso8859162001", kCFStringEncodingISOLatin10, NoEncodingFlags },
+    { "isoir226", kCFStringEncodingISOLatin10, NoEncodingFlags },
+    { "l10", kCFStringEncodingISOLatin10, NoEncodingFlags },
+    { "latin10", kCFStringEncodingISOLatin10, NoEncodingFlags },
     { "cp437", kCFStringEncodingDOSLatinUS, NoEncodingFlags },
     { "437", kCFStringEncodingDOSLatinUS, NoEncodingFlags },
     { "cspc8codepage437", kCFStringEncodingDOSLatinUS, NoEncodingFlags },
@@ -230,27 +233,8 @@ static const CharsetEntry table[] = {
     { "JIS_C6226-1978", kCFStringEncodingJIS_C6226_78, IsJapanese },
     { "csiso42jisc62261978", kCFStringEncodingJIS_C6226_78, IsJapanese },
     { "isoir42", kCFStringEncodingJIS_C6226_78, IsJapanese },
-    { "Shift_JIS", kCFStringEncodingShiftJIS_X0213_00, IsJapanese },
-    { "csshiftjis", kCFStringEncodingShiftJIS_X0213_00, IsJapanese },
-    { "mskanji", kCFStringEncodingShiftJIS_X0213_00, IsJapanese },
-    { "sjis", kCFStringEncodingShiftJIS_X0213_00, IsJapanese },
-    { "xsjis", kCFStringEncodingShiftJIS_X0213_00, IsJapanese },
+    { "Shift_JIS_X0213-2000", kCFStringEncodingShiftJIS_X0213_00, IsJapanese },
     { "GB18030", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "chinese", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "cngb", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "cp936", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "csgb2312", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "csgb231280", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "csiso58gb231280", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "euccn", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "gb2312", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "gb231280", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "gbk", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "isoir58", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "ms936", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "windows936", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "xeuccn", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
-    { "xgbk", kCFStringEncodingGB_18030_2000, NoEncodingFlags },
     { "ISO-2022-JP", kCFStringEncodingISO_2022_JP, IsJapanese },
     { "csiso2022jp", kCFStringEncodingISO_2022_JP, IsJapanese },
     { "jis7", kCFStringEncodingISO_2022_JP, IsJapanese },
@@ -267,27 +251,48 @@ static const CharsetEntry table[] = {
     { "extendedunixcodepackedformatforjapanese", kCFStringEncodingEUC_JP, IsJapanese },
     { "xeuc", kCFStringEncodingEUC_JP, IsJapanese },
     { "xeucjp", kCFStringEncodingEUC_JP, IsJapanese },
+    { "EUC-CN", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "chinese", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "cngb", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "cp936", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "csgb2312", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "csgb231280", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "csiso58gb231280", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "gb2312", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "gb231280", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "gbk", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "isoir58", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "ms936", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "windows936", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "xeuccn", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
+    { "xgbk", kCFStringEncodingEUC_CN_DOSVariant, NoEncodingFlags },
     { "EUC-TW", kCFStringEncodingEUC_TW, NoEncodingFlags },
-    { "EUC-KR", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "cp949", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "cseuckr", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "csksc56011987", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "isoir149", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "korean", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "ksc5601", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "ksc56011987", kCFStringEncodingEUC_KR, NoEncodingFlags },
-    { "ksc56011989", kCFStringEncodingEUC_KR, NoEncodingFlags },
+    { "EUC-KR", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "cp949", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "cseuckr", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "csksc56011987", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "isoir149", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "korean", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "ksc5601", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "ksc56011987", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "ksc56011989", kCFStringEncodingEUC_KR_DOSVariant, NoEncodingFlags },
+    { "Shift_JIS", kCFStringEncodingShiftJIS_DOSVariant, IsJapanese },
+    { "csshiftjis", kCFStringEncodingShiftJIS_DOSVariant, IsJapanese },
+    { "mskanji", kCFStringEncodingShiftJIS_DOSVariant, IsJapanese },
+    { "sjis", kCFStringEncodingShiftJIS_DOSVariant, IsJapanese },
+    { "xsjis", kCFStringEncodingShiftJIS_DOSVariant, IsJapanese },
     { "KOI8-R", kCFStringEncodingKOI8_R, NoEncodingFlags },
     { "cskoi8r", kCFStringEncodingKOI8_R, NoEncodingFlags },
     { "koi", kCFStringEncodingKOI8_R, NoEncodingFlags },
     { "koi8", kCFStringEncodingKOI8_R, NoEncodingFlags },
+    { "Big5", kCFStringEncodingBig5_DOSVariant, NoEncodingFlags },
+    { "cnbig5", kCFStringEncodingBig5_DOSVariant, NoEncodingFlags },
+    { "csbig5", kCFStringEncodingBig5_DOSVariant, NoEncodingFlags },
+    { "xxbig5", kCFStringEncodingBig5_DOSVariant, NoEncodingFlags },
     { "x-mac-roman-latin1", kCFStringEncodingMacRomanLatin1, NoEncodingFlags },
     { "HZ-GB-2312", kCFStringEncodingHZ_GB_2312, NoEncodingFlags },
-    { "Big5", kCFStringEncodingBig5_HKSCS_1999, NoEncodingFlags },
-    { "big5hkscs", kCFStringEncodingBig5_HKSCS_1999, NoEncodingFlags },
-    { "cnbig5", kCFStringEncodingBig5_HKSCS_1999, NoEncodingFlags },
-    { "csbig5", kCFStringEncodingBig5_HKSCS_1999, NoEncodingFlags },
-    { "xxbig5", kCFStringEncodingBig5_HKSCS_1999, NoEncodingFlags },
+    { "Big5-HKSCS", kCFStringEncodingBig5_HKSCS_1999, NoEncodingFlags },
+    { "KOI8-U", kCFStringEncodingKOI8_U, NoEncodingFlags },
     { "cp037", kCFStringEncodingEBCDIC_CP037, NoEncodingFlags },
     { "csibm037", kCFStringEncodingEBCDIC_CP037, NoEncodingFlags },
     { "ebcdiccpca", kCFStringEncodingEBCDIC_CP037, NoEncodingFlags },
