@@ -72,8 +72,7 @@ public:
     QString ref() const;
     bool hasRef() const;
 
-    QString htmlRef() const;
-    QString encodedHtmlRef() const;
+    QString encodedHtmlRef() const { return ref(); }
 
     void setProtocol(const QString &);
     void setHost(const QString &);
@@ -81,15 +80,17 @@ public:
     void setUser(const QString &);
     void setPass(const QString &);
     void setPath(const QString &);
-    void setQuery(const QString &, int encoding_hint=0);
+    void setQuery(const QString &);
     void setRef(const QString &);
 
     QString prettyURL() const;
     
     NSURL *getNSURL() const;
     NSData *getNSData() const;
-    
-    static QString decode_string(const QString &);
+
+    bool isLocalFile() const;
+
+    static QString decode_string(const QString &, const QTextCodec * = 0);
     static QString encode_string(const QString &);
     
     friend bool operator==(const KURL &, const KURL &);
