@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (C) 2004 Apple Computer, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,8 +32,17 @@
 #import "KWQView.h"
 #import "WebCoreBridge.h"
 #import "WebCoreScrollView.h"
+#import "WebCoreTextRenderer.h"
+#import "WebCoreTextRendererFactory.h"
 
-#define MIN_LINES 4 /* ensures we have a scroll bar */
+@interface NSTableView (KWQListBoxKnowsAppKitSecrets)
+- (NSCell *)_accessibilityTableCell:(int)row tableColumn:(NSTableColumn *)tableColumn;
+@end
+
+const int minLines = 4; /* ensures we have a scroll bar */
+const float bottomMargin = 1;
+const float leftMargin = 2;
+const float rightMargin = 2;
 
 @interface KWQListBoxScrollView : WebCoreScrollView
 @end
@@ -41,13 +50,12 @@
 @interface KWQTableView : NSTableView <KWQWidgetHolder>
 {
     QListBox *_box;
-    NSArray *_items;
     BOOL processingMouseEvent;
     BOOL clickedDuringMouseEvent;
     BOOL inNextValidKeyView;
     NSWritingDirection _direction;
 }
-- initWithListBox:(QListBox *)b items:(NSArray *)items;
+- (id)initWithListBox:(QListBox *)b;
 - (void)_KWQ_setKeyboardFocusRingNeedsDisplay;
 - (QWidget *)widget;
 - (void)setBaseWritingDirection:(NSWritingDirection)direction;
@@ -66,36 +74,34 @@ static NSFont *groupLabelFont()
     return font;
 }
 
-static NSParagraphStyle *paragraphStyle(NSWritingDirection direction)
+static id <WebCoreTextRenderer> itemTextRenderer()
 {
-    static NSParagraphStyle *leftStyle;
-    static NSParagraphStyle *rightStyle;
-    NSParagraphStyle **style = direction == NSWritingDirectionRightToLeft ? &rightStyle : &leftStyle;
-    if (*style == nil) {
-        NSMutableParagraphStyle *mutableStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
-        [mutableStyle setBaseWritingDirection:direction];
-        *style = [mutableStyle copy];
-        [mutableStyle release];
+    if ([NSGraphicsContext currentContextDrawingToScreen]) {
+        static id <WebCoreTextRenderer> renderer = [[WebCoreTextRendererFactory sharedFactory]
+            rendererWithFont:itemFont() usingPrinterFont:NO];
+        return renderer;
+    } else {
+        static id <WebCoreTextRenderer> renderer = [[WebCoreTextRendererFactory sharedFactory]
+            rendererWithFont:itemFont() usingPrinterFont:YES];
+        return renderer;
     }
-    return *style;
 }
 
-static NSDictionary *stringAttributes(NSWritingDirection direction, bool isGroupLabel)
+static id <WebCoreTextRenderer> groupLabelTextRenderer()
 {
-    static NSDictionary *attributeGlobals[4];
-    NSDictionary **attributes = &attributeGlobals[(direction == NSWritingDirectionRightToLeft ? 0 : 1) + (isGroupLabel ? 0 : 2)];
-    if (*attributes == nil) {
-        *attributes = [[NSDictionary dictionaryWithObjectsAndKeys:
-            isGroupLabel ? groupLabelFont() : itemFont(), NSFontAttributeName,
-            paragraphStyle(direction), NSParagraphStyleAttributeName,
-            nil] retain];
+    if ([NSGraphicsContext currentContextDrawingToScreen]) {
+        static id <WebCoreTextRenderer> renderer = [[WebCoreTextRendererFactory sharedFactory]
+            rendererWithFont:groupLabelFont() usingPrinterFont:NO];
+        return renderer;
+    } else {
+        static id <WebCoreTextRenderer> renderer = [[WebCoreTextRendererFactory sharedFactory]
+            rendererWithFont:groupLabelFont() usingPrinterFont:YES];
+        return renderer;
     }
-    return *attributes;
 }
 
 QListBox::QListBox(QWidget *parent)
     : QScrollView(parent)
-    , _insertingItems(false)
     , _changingSelection(false)
     , _enabled(true)
     , _widthGood(false)
@@ -104,7 +110,6 @@ QListBox::QListBox(QWidget *parent)
 {
     KWQ_BLOCK_EXCEPTIONS;
 
-    _items = [[NSMutableArray alloc] init];
     NSScrollView *scrollView = [[KWQListBoxScrollView alloc] init];
     setView(scrollView);
     [scrollView release];
@@ -124,7 +129,7 @@ QListBox::QListBox(QWidget *parent)
     // <rdar://problem/3226083>: REGRESSION (Panther): white box overlaying select lists at nvidia.com drivers page
     [[scrollView contentView] releaseGState];
     
-    KWQTableView *tableView = [[KWQTableView alloc] initWithListBox:this items:_items];
+    KWQTableView *tableView = [[KWQTableView alloc] initWithListBox:this];
     [scrollView setDocumentView:tableView];
     [tableView release];
     [scrollView setVerticalLineScroll:[tableView rowHeight]];
@@ -140,30 +145,13 @@ QListBox::~QListBox()
     NSTableView *tableView = [scrollView documentView];
     [tableView setDelegate:nil];
     [tableView setDataSource:nil];
-    [_items release];
     KWQ_UNBLOCK_EXCEPTIONS;
 }
 
-uint QListBox::count() const
-{
-    KWQ_BLOCK_EXCEPTIONS;
-    return [_items count];
-    KWQ_UNBLOCK_EXCEPTIONS;
-
-    return 0;
-}
-
 void QListBox::clear()
 {
-    KWQ_BLOCK_EXCEPTIONS;
-    [_items removeAllObjects];
-    if (!_insertingItems) {
-        NSScrollView *scrollView = getView();
-        NSTableView *tableView = [scrollView documentView];
-        [tableView reloadData];
-    }
-    KWQ_UNBLOCK_EXCEPTIONS;
-    _widthGood = NO;
+    _items.clear();
+    _widthGood = false;
 }
 
 void QListBox::setSelectionMode(SelectionMode mode)
@@ -176,47 +164,14 @@ void QListBox::setSelectionMode(SelectionMode mode)
     KWQ_UNBLOCK_EXCEPTIONS;
 }
 
-void QListBox::insertItem(const QString &text, int index, bool isLabel)
+void QListBox::appendItem(const QString &text, bool isLabel)
 {
-    ASSERT(index >= 0);
-
-    KWQ_BLOCK_EXCEPTIONS;
-
-    NSScrollView *scrollView = getView();
-    KWQTableView *tableView = [scrollView documentView];
-
-    NSAttributedString *s = [[NSAttributedString alloc] initWithString:text.getNSString()
-        attributes:stringAttributes([tableView baseWritingDirection], isLabel)];
-
-    int c = count();
-    if (index >= c) {
-        [_items addObject:s];
-    } else {
-        [_items replaceObjectAtIndex:index withObject:s];
-    }
- 
-    [s release];
-
-    if (!_insertingItems) {
-        [tableView reloadData];
-    }
-
-    KWQ_UNBLOCK_EXCEPTIONS;
-
-    _widthGood = NO;
+    _items.append(KWQListBoxItem(text, isLabel));
+    _widthGood = false;
 }
 
-void QListBox::beginBatchInsert()
+void QListBox::doneAppendingItems()
 {
-    ASSERT(!_insertingItems);
-    _insertingItems = true;
-}
-
-void QListBox::endBatchInsert()
-{
-    ASSERT(_insertingItems);
-    _insertingItems = false;
-
     KWQ_BLOCK_EXCEPTIONS;
 
     NSScrollView *scrollView = getView();
@@ -229,7 +184,6 @@ void QListBox::endBatchInsert()
 void QListBox::setSelected(int index, bool selectIt)
 {
     ASSERT(index >= 0);
-    ASSERT(!_insertingItems);
 
     KWQ_BLOCK_EXCEPTIONS;
 
@@ -251,7 +205,6 @@ void QListBox::setSelected(int index, bool selectIt)
 bool QListBox::isSelected(int index) const
 {
     ASSERT(index >= 0);
-    ASSERT(!_insertingItems);
 
     KWQ_BLOCK_EXCEPTIONS;
 
@@ -266,12 +219,19 @@ bool QListBox::isSelected(int index) const
 
 void QListBox::setEnabled(bool enabled)
 {
-    _enabled = enabled;
-    // You would think this would work, but not until AK fixes 2177792
-    //KWQ_BLOCK_EXCEPTIONS;
-    //NSTableView *tableView = [(NSScrollView *)getView() documentView];
-    //[tableView setEnabled:enabled];
-    //KWQ_UNBLOCK_EXCEPTIONS;
+    if (enabled != _enabled) {
+        // You would think this would work, but not until AppKit bug 2177792 if fixed.
+        //KWQ_BLOCK_EXCEPTIONS;
+        //NSTableView *tableView = [(NSScrollView *)getView() documentView];
+        //[tableView setEnabled:enabled];
+        //KWQ_UNBLOCK_EXCEPTIONS;
+
+        _enabled = enabled;
+
+        NSScrollView *scrollView = getView();
+        NSTableView *tableView = [scrollView documentView];
+        [tableView reloadData];
+    }
 }
 
 bool QListBox::isEnabled()
@@ -281,37 +241,44 @@ bool QListBox::isEnabled()
 
 QSize QListBox::sizeForNumberOfLines(int lines) const
 {
-    ASSERT(!_insertingItems);
-
-    NSScrollView *scrollView = getView();
-
     NSSize size = {0,0};
 
     KWQ_BLOCK_EXCEPTIONS;
-    NSTableView *tableView = [scrollView documentView];
+
+    NSScrollView *scrollView = getView();
+    KWQTableView *tableView = [scrollView documentView];
     
-    float width;
-    if (_widthGood) {
-        width = _width;
-    } else {
-        width = 0;
-        NSCell *cell = [[[tableView tableColumns] objectAtIndex:0] dataCell];
-        NSEnumerator *e = [_items objectEnumerator];
-        NSString *text;
-        while ((text = [e nextObject])) {
-            [cell setStringValue:text];
-            NSSize size = [cell cellSize];
-            width = MAX(width, size.width);
+    if (!_widthGood) {
+        float width = 0;
+        QValueListConstIterator<KWQListBoxItem> i = const_cast<const QValueList<KWQListBoxItem> &>(_items).begin();
+        QValueListConstIterator<KWQListBoxItem> e = const_cast<const QValueList<KWQListBoxItem> &>(_items).end();
+        if (i != e) {
+            WebCoreTextStyle style;
+            WebCoreInitializeEmptyTextStyle(&style);
+            style.rtl = [tableView baseWritingDirection] == NSWritingDirectionRightToLeft;
+            style.applyRunRounding = NO;
+            style.applyWordRounding = NO;
+            do {
+                const QString &s = (*i).string;
+                id <WebCoreTextRenderer> renderer = (*i).isGroupLabel ? groupLabelTextRenderer() : itemTextRenderer();
+                ++i;
+
+                WebCoreTextRun run;
+                int length = s.length();
+                WebCoreInitializeTextRun(&run, reinterpret_cast<const UniChar *>(s.unicode()), length, 0, length);
+
+                float textWidth = [renderer floatWidthForRun:&run style:&style widths:0];
+                width = kMax(width, textWidth);
+            } while (i != e);
         }
-        _width = width;
-        _widthGood = YES;
+        _width = ceilf(width);
+        _widthGood = true;
     }
     
-    NSSize contentSize;
-    contentSize.width = ceil(width);
-    contentSize.height = ceil(([tableView rowHeight] + [tableView intercellSpacing].height) * MAX(MIN_LINES, lines));
-    size = [NSScrollView frameSizeForContentSize:contentSize
+    size = [NSScrollView frameSizeForContentSize:NSMakeSize(_width, [tableView rowHeight] * MAX(minLines, lines))
         hasHorizontalScroller:NO hasVerticalScroller:YES borderType:NSBezelBorder];
+    size.width += [NSScroller scrollerWidthForControlSize:NSSmallControlSize] - [NSScroller scrollerWidth] + leftMargin + rightMargin;
+
     KWQ_UNBLOCK_EXCEPTIONS;
 
     return QSize(size);
@@ -321,9 +288,7 @@ QWidget::FocusPolicy QListBox::focusPolicy() const
 {
     KWQ_BLOCK_EXCEPTIONS;
     
-    // Add an additional check here.
-    // For now, selects are only focused when full
-    // keyboard access is turned on.
+    // Lists are only focused when full keyboard access is turned on.
     unsigned keyboardUIMode = [KWQKHTMLPart::bridgeForWidget(this) keyboardUIMode];
     if ((keyboardUIMode & WebCoreKeyboardAccessFull) == 0)
         return NoFocus;
@@ -346,14 +311,6 @@ void QListBox::setWritingDirection(QPainter::TextDirection d)
     KWQTableView *tableView = [scrollView documentView];
     NSWritingDirection direction = d == QPainter::RTL ? NSWritingDirectionRightToLeft : NSWritingDirectionLeftToRight;
     if ([tableView baseWritingDirection] != direction) {
-        int n = count();
-        for (int i = 0; i < n; i++) {
-            NSAttributedString *o = [_items objectAtIndex:i];
-            NSAttributedString *s = [[NSAttributedString alloc] initWithString:[o string]
-                attributes:stringAttributes([tableView baseWritingDirection], itemIsGroupLabel(i))];
-            [_items replaceObjectAtIndex:i withObject:s];
-            [s release];
-        }
         [tableView setBaseWritingDirection:direction];
         [tableView reloadData];
     }
@@ -361,21 +318,6 @@ void QListBox::setWritingDirection(QPainter::TextDirection d)
     KWQ_UNBLOCK_EXCEPTIONS;
 }
 
-bool QListBox::itemIsGroupLabel(int index) const
-{
-    ASSERT(index >= 0);
-
-    KWQ_BLOCK_EXCEPTIONS;
-
-    NSAttributedString *s = [_items objectAtIndex:index];
-    NSFont *f = [s attribute:NSFontAttributeName atIndex:0 effectiveRange:NULL];
-    return f == groupLabelFont();
-
-    KWQ_UNBLOCK_EXCEPTIONS;
-
-    return false;
-}
-
 @implementation KWQListBoxScrollView
 
 - (void)setFrameSize:(NSSize)size
@@ -399,16 +341,15 @@ bool QListBox::itemIsGroupLabel(int index) const
 
 @implementation KWQTableView
 
-- initWithListBox:(QListBox *)b items:(NSArray *)i
+- (id)initWithListBox:(QListBox *)b
 {
     [super init];
+
     _box = b;
-    _items = i;
 
     NSTableColumn *column = [[NSTableColumn alloc] initWithIdentifier:nil];
 
     [column setEditable:NO];
-    [[column dataCell] setFont:itemFont()];
 
     [self addTableColumn:column];
 
@@ -417,7 +358,7 @@ bool QListBox::itemIsGroupLabel(int index) const
     [self setAllowsMultipleSelection:NO];
     [self setHeaderView:nil];
     [self setIntercellSpacing:NSMakeSize(0, 0)];
-    [self setRowHeight:ceil([[column dataCell] cellSize].height)];
+    [self setRowHeight:ceilf([itemFont() ascender] - [itemFont() descender] + bottomMargin)];
     
     [self setDataSource:self];
     [self setDelegate:self];
@@ -425,7 +366,7 @@ bool QListBox::itemIsGroupLabel(int index) const
     return self;
 }
 
--(void)mouseDown:(NSEvent *)event
+- (void)mouseDown:(NSEvent *)event
 {
     processingMouseEvent = TRUE;
     [super mouseDown:event];
@@ -480,21 +421,31 @@ bool QListBox::itemIsGroupLabel(int index) const
     return resign;
 }
 
--(NSView *)nextKeyView
+- (BOOL)canBecomeKeyView {
+    // Simplified method from NSView; overridden to replace NSView's way of checking
+    // for full keyboard access with ours.
+    if (!KWQKHTMLPart::partForWidget(_box)->tabsToAllControls()) {
+        return NO;
+    }
+    
+    return ([self window] != nil) && ![self isHiddenOrHasHiddenAncestor] && [self acceptsFirstResponder];
+}
+
+- (NSView *)nextKeyView
 {
     return _box && inNextValidKeyView
         ? KWQKHTMLPart::nextKeyViewForWidget(_box, KWQSelectingNext)
         : [super nextKeyView];
 }
 
--(NSView *)previousKeyView
+- (NSView *)previousKeyView
 {
     return _box && inNextValidKeyView
         ? KWQKHTMLPart::nextKeyViewForWidget(_box, KWQSelectingPrevious)
         : [super previousKeyView];
 }
 
--(NSView *)nextValidKeyView
+- (NSView *)nextValidKeyView
 {
     inNextValidKeyView = YES;
     NSView *view = [super nextValidKeyView];
@@ -502,7 +453,7 @@ bool QListBox::itemIsGroupLabel(int index) const
     return view;
 }
 
--(NSView *)previousValidKeyView
+- (NSView *)previousValidKeyView
 {
     inNextValidKeyView = YES;
     NSView *view = [super previousValidKeyView];
@@ -512,12 +463,12 @@ bool QListBox::itemIsGroupLabel(int index) const
 
 - (int)numberOfRowsInTableView:(NSTableView *)tableView
 {
-    return [_items count];
+    return _box->count();
 }
 
 - (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)column row:(int)row
 {
-    return [_items objectAtIndex:row];
+    return nil;
 }
 
 - (void)tableViewSelectionDidChange:(NSNotification *)notification
@@ -534,7 +485,7 @@ bool QListBox::itemIsGroupLabel(int index) const
 
 - (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(int)row
 {
-    return !_box->itemIsGroupLabel(row);
+    return !_box->itemAtIndex(row).isGroupLabel;
 }
 
 - (BOOL)selectionShouldChangeInTableView:(NSTableView *)aTableView
@@ -542,10 +493,50 @@ bool QListBox::itemIsGroupLabel(int index) const
     return _box->isEnabled();
 }
 
-- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row
+- (void)drawRow:(int)row clipRect:(NSRect)clipRect
 {
-    ASSERT([cell isKindOfClass:[NSCell class]]);
-    [(NSCell *)cell setEnabled:_box->isEnabled()];
+    const KWQListBoxItem &item = _box->itemAtIndex(row);
+
+    NSColor *color;
+    if (_box->isEnabled()) {
+        if ([self isRowSelected:row] && [[self window] firstResponder] == self && ([[self window] isKeyWindow] || ![[self window] canBecomeKeyWindow])) {
+            color = [NSColor alternateSelectedControlTextColor];
+        } else {
+            color = [NSColor controlTextColor];
+        }
+    } else {
+        color = [NSColor disabledControlTextColor];
+    }
+
+    bool RTL = _direction == NSWritingDirectionRightToLeft;
+
+    id <WebCoreTextRenderer> renderer = item.isGroupLabel ? groupLabelTextRenderer() : itemTextRenderer();
+
+    WebCoreTextStyle style;
+    WebCoreInitializeEmptyTextStyle(&style);
+    style.rtl = RTL;
+    style.applyRunRounding = NO;
+    style.applyWordRounding = NO;
+    style.textColor = color;
+
+    WebCoreTextRun run;
+    int length = item.string.length();
+    WebCoreInitializeTextRun(&run, reinterpret_cast<const UniChar *>(item.string.unicode()), length, 0, length);
+
+    NSRect cellRect = [self frameOfCellAtColumn:0 row:row];
+    NSPoint point;
+    if (!RTL) {
+        point.x = NSMinX(cellRect) + leftMargin;
+    } else {
+        point.x = NSMaxX(cellRect) - rightMargin - [renderer floatWidthForRun:&run style:&style widths:0];
+    }
+    point.y = NSMaxY(cellRect) + [itemFont() descender] - bottomMargin;
+
+    WebCoreTextGeometry geometry;
+    WebCoreInitializeEmptyTextGeometry(&geometry);
+    geometry.point = point;
+    
+    [renderer drawRun:&run style:&style geometry:&geometry];
 }
 
 - (void)_KWQ_setKeyboardFocusRingNeedsDisplay
@@ -568,4 +559,11 @@ bool QListBox::itemIsGroupLabel(int index) const
     return _direction;
 }
 
+- (NSCell *)_accessibilityTableCell:(int)row tableColumn:(NSTableColumn *)tableColumn
+{
+    NSCell *cell = [super _accessibilityTableCell:row tableColumn:tableColumn];
+    [cell setStringValue:_box->itemAtIndex(row).string.getNSString()];
+    return cell;
+}
+
 @end
