@@ -29,25 +29,22 @@ class QLineEdit;
 @class KWQTextFieldFormatter;
 @protocol KWQWidgetHolder;
 
-@interface KWQTextField : NSTextField <KWQWidgetHolder>
+@interface KWQTextFieldController : NSObject
 {
 @private
+    NSTextField* field;
     QLineEdit *widget;
-    NSTextField *secureField;
     KWQTextFieldFormatter *formatter;
     BOOL hasFocus;
     BOOL edited;
-    BOOL inNextValidKeyView;
     NSRange lastSelectedRange;
     BOOL inDrawingMachinery;
+    NSWritingDirection baseWritingDirection;
 }
 
-- (id)initWithQLineEdit:(QLineEdit *)widget;
-
 - (void)invalidate;
 
-- (void)setPasswordMode:(BOOL)flag;
-- (BOOL)passwordMode;
+- (void)setHasFocus:(BOOL)hasFocus;
 
 - (void)setMaximumLength:(int)len;
 - (int)maximumLength;
@@ -56,5 +53,44 @@ class QLineEdit;
 - (BOOL)edited;
 
 - (void)setBaseWritingDirection:(NSWritingDirection)direction;
+- (NSWritingDirection)baseWritingDirection;
 
 @end
+
+@interface KWQTextField : NSTextField <KWQWidgetHolder>
+{
+@private
+    KWQTextFieldController* controller;
+    BOOL inNextValidKeyView;
+}
+
+- (id)initWithQLineEdit:(QLineEdit *)widget;
+- (KWQTextFieldController *)controller;
+
+@end
+
+@interface KWQSecureTextField : NSSecureTextField <KWQWidgetHolder>
+{
+@private
+    KWQTextFieldController* controller;
+    BOOL inNextValidKeyView;
+    BOOL inSetFrameSize;
+}
+
+- (id)initWithQLineEdit:(QLineEdit *)widget;
+- (KWQTextFieldController *)controller;
+
+@end
+
+@interface KWQSearchField : NSSearchField <KWQWidgetHolder>
+{
+@private
+    KWQTextFieldController* controller;
+    BOOL inNextValidKeyView;
+}
+
+- (id)initWithQLineEdit:(QLineEdit *)widget;
+- (KWQTextFieldController *)controller;
+
+@end
+
