@@ -32,8 +32,12 @@
 - (NSSize)size;
 - (void)resize:(NSSize)s;
 - (void)drawImageInRect:(NSRect)ir fromRect:(NSRect)fr;
+- (void)drawImageInRect:(NSRect)ir fromRect:(NSRect)fr compositeOperator:(NSCompositingOperation)compsiteOperator context:(CGContextRef)context;
 - (void)stopAnimation;
-- (void)tileInRect:(NSRect)r fromPoint:(NSPoint)p;
+- (void)tileInRect:(NSRect)r fromPoint:(NSPoint)p context:(CGContextRef)context;
 - (BOOL)isNull;
 - (id <WebCoreImageRenderer>)retainOrCopyIfNeeded;
+- (void)increaseUseCount;
+- (void)decreaseUseCount;
+- (void)flushRasterCache;
 @end
